# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _operators
else:
    import _operators

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _operators.SWIG_PyInstanceMethod_New
_swig_new_static_method = _operators.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._ser.mem_manager
import mfem._ser.vector
import mfem._ser.array
class Operator(object):
    r"""Proxy of C++ mfem::Operator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DIAG_ZERO = _operators.Operator_DIAG_ZERO
    
    DIAG_ONE = _operators.Operator_DIAG_ONE
    
    DIAG_KEEP = _operators.Operator_DIAG_KEEP
    

    def InitTVectors(self, Po, Ri, Pi, x, b, X, B):
        r"""InitTVectors(Operator self, Operator Po, Operator Ri, Operator Pi, Vector x, Vector b, Vector X, Vector B)"""
        return _operators.Operator_InitTVectors(self, Po, Ri, Pi, x, b, X, B)
    InitTVectors = _swig_new_instance_method(_operators.Operator_InitTVectors)

    def __init__(self, *args):
        r"""
        __init__(Operator self, int s=0) -> Operator
        __init__(Operator self, int h, int w) -> Operator
        """
        if self.__class__ == Operator:
            _self = None
        else:
            _self = self
        _operators.Operator_swiginit(self, _operators.new_Operator(_self, *args))

    def Height(self):
        r"""Height(Operator self) -> int"""
        return _operators.Operator_Height(self)
    Height = _swig_new_instance_method(_operators.Operator_Height)

    def NumRows(self):
        r"""NumRows(Operator self) -> int"""
        return _operators.Operator_NumRows(self)
    NumRows = _swig_new_instance_method(_operators.Operator_NumRows)

    def Width(self):
        r"""Width(Operator self) -> int"""
        return _operators.Operator_Width(self)
    Width = _swig_new_instance_method(_operators.Operator_Width)

    def NumCols(self):
        r"""NumCols(Operator self) -> int"""
        return _operators.Operator_NumCols(self)
    NumCols = _swig_new_instance_method(_operators.Operator_NumCols)

    def GetMemoryClass(self):
        r"""GetMemoryClass(Operator self) -> mfem::MemoryClass"""
        return _operators.Operator_GetMemoryClass(self)
    GetMemoryClass = _swig_new_instance_method(_operators.Operator_GetMemoryClass)

    def Mult(self, x, y):
        r"""Mult(Operator self, Vector x, Vector y)"""
        return _operators.Operator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.Operator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(Operator self, Vector x, Vector y)"""
        return _operators.Operator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_operators.Operator_MultTranspose)

    def GetGradient(self, x):
        r"""GetGradient(Operator self, Vector x) -> Operator"""
        return _operators.Operator_GetGradient(self, x)
    GetGradient = _swig_new_instance_method(_operators.Operator_GetGradient)

    def GetProlongation(self):
        r"""GetProlongation(Operator self) -> Operator"""
        return _operators.Operator_GetProlongation(self)
    GetProlongation = _swig_new_instance_method(_operators.Operator_GetProlongation)

    def GetRestriction(self):
        r"""GetRestriction(Operator self) -> Operator"""
        return _operators.Operator_GetRestriction(self)
    GetRestriction = _swig_new_instance_method(_operators.Operator_GetRestriction)

    def GetOutputProlongation(self):
        r"""GetOutputProlongation(Operator self) -> Operator"""
        return _operators.Operator_GetOutputProlongation(self)
    GetOutputProlongation = _swig_new_instance_method(_operators.Operator_GetOutputProlongation)

    def GetOutputRestriction(self):
        r"""GetOutputRestriction(Operator self) -> Operator"""
        return _operators.Operator_GetOutputRestriction(self)
    GetOutputRestriction = _swig_new_instance_method(_operators.Operator_GetOutputRestriction)

    def FormLinearSystem(self, ess_tdof_list, x, b, A, X, B, copy_interior=0):
        r"""FormLinearSystem(Operator self, intArray ess_tdof_list, Vector x, Vector b, mfem::Operator *& A, Vector X, Vector B, int copy_interior=0)"""
        return _operators.Operator_FormLinearSystem(self, ess_tdof_list, x, b, A, X, B, copy_interior)
    FormLinearSystem = _swig_new_instance_method(_operators.Operator_FormLinearSystem)

    def FormRectangularLinearSystem(self, trial_tdof_list, test_tdof_list, x, b, A, X, B):
        r"""FormRectangularLinearSystem(Operator self, intArray trial_tdof_list, intArray test_tdof_list, Vector x, Vector b, mfem::Operator *& A, Vector X, Vector B)"""
        return _operators.Operator_FormRectangularLinearSystem(self, trial_tdof_list, test_tdof_list, x, b, A, X, B)
    FormRectangularLinearSystem = _swig_new_instance_method(_operators.Operator_FormRectangularLinearSystem)

    def RecoverFEMSolution(self, X, b, x):
        r"""RecoverFEMSolution(Operator self, Vector X, Vector b, Vector x)"""
        return _operators.Operator_RecoverFEMSolution(self, X, b, x)
    RecoverFEMSolution = _swig_new_instance_method(_operators.Operator_RecoverFEMSolution)

    def FormSystemOperator(self, ess_tdof_list, A):
        r"""FormSystemOperator(Operator self, intArray ess_tdof_list, mfem::Operator *& A)"""
        return _operators.Operator_FormSystemOperator(self, ess_tdof_list, A)
    FormSystemOperator = _swig_new_instance_method(_operators.Operator_FormSystemOperator)

    def FormRectangularSystemOperator(self, trial_tdof_list, test_tdof_list, A):
        r"""FormRectangularSystemOperator(Operator self, intArray trial_tdof_list, intArray test_tdof_list, mfem::Operator *& A)"""
        return _operators.Operator_FormRectangularSystemOperator(self, trial_tdof_list, test_tdof_list, A)
    FormRectangularSystemOperator = _swig_new_instance_method(_operators.Operator_FormRectangularSystemOperator)

    def FormDiscreteOperator(self, A):
        r"""FormDiscreteOperator(Operator self, mfem::Operator *& A)"""
        return _operators.Operator_FormDiscreteOperator(self, A)
    FormDiscreteOperator = _swig_new_instance_method(_operators.Operator_FormDiscreteOperator)
    __swig_destroy__ = _operators.delete_Operator
    ANY_TYPE = _operators.Operator_ANY_TYPE
    
    MFEM_SPARSEMAT = _operators.Operator_MFEM_SPARSEMAT
    
    Hypre_ParCSR = _operators.Operator_Hypre_ParCSR
    
    PETSC_MATAIJ = _operators.Operator_PETSC_MATAIJ
    
    PETSC_MATIS = _operators.Operator_PETSC_MATIS
    
    PETSC_MATSHELL = _operators.Operator_PETSC_MATSHELL
    
    PETSC_MATNEST = _operators.Operator_PETSC_MATNEST
    
    PETSC_MATHYPRE = _operators.Operator_PETSC_MATHYPRE
    
    PETSC_MATGENERIC = _operators.Operator_PETSC_MATGENERIC
    
    Complex_Operator = _operators.Operator_Complex_Operator
    
    MFEM_ComplexSparseMat = _operators.Operator_MFEM_ComplexSparseMat
    
    Complex_Hypre_ParCSR = _operators.Operator_Complex_Hypre_ParCSR
    

    def GetType(self):
        r"""GetType(Operator self) -> mfem::Operator::Type"""
        return _operators.Operator_GetType(self)
    GetType = _swig_new_instance_method(_operators.Operator_GetType)

    def PrintMatlab(self, *args):
        r"""
        PrintMatlab(Operator self, std::ostream & out, int n=0, int m=0)
        PrintMatlab(Operator self, char const * file, int precision=8)
        """
        return _operators.Operator_PrintMatlab(self, *args)
    PrintMatlab = _swig_new_instance_method(_operators.Operator_PrintMatlab)
    def __disown__(self):
        self.this.disown()
        _operators.disown_Operator(self)
        return weakref.proxy(self)

# Register Operator in _operators:
_operators.Operator_swigregister(Operator)

class TimeDependentOperator(Operator):
    r"""Proxy of C++ mfem::TimeDependentOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    EXPLICIT = _operators.TimeDependentOperator_EXPLICIT
    
    IMPLICIT = _operators.TimeDependentOperator_IMPLICIT
    
    HOMOGENEOUS = _operators.TimeDependentOperator_HOMOGENEOUS
    
    NORMAL = _operators.TimeDependentOperator_NORMAL
    
    ADDITIVE_TERM_1 = _operators.TimeDependentOperator_ADDITIVE_TERM_1
    
    ADDITIVE_TERM_2 = _operators.TimeDependentOperator_ADDITIVE_TERM_2
    

    def __init__(self, *args):
        r"""
        __init__(TimeDependentOperator self, int n=0, double t_=0.0, mfem::TimeDependentOperator::Type type_=EXPLICIT) -> TimeDependentOperator
        __init__(TimeDependentOperator self, int h, int w, double t_=0.0, mfem::TimeDependentOperator::Type type_=EXPLICIT) -> TimeDependentOperator
        """
        if self.__class__ == TimeDependentOperator:
            _self = None
        else:
            _self = self
        _operators.TimeDependentOperator_swiginit(self, _operators.new_TimeDependentOperator(_self, *args))

    def GetTime(self):
        r"""GetTime(TimeDependentOperator self) -> double"""
        return _operators.TimeDependentOperator_GetTime(self)
    GetTime = _swig_new_instance_method(_operators.TimeDependentOperator_GetTime)

    def SetTime(self, _t):
        r"""SetTime(TimeDependentOperator self, double const _t)"""
        return _operators.TimeDependentOperator_SetTime(self, _t)
    SetTime = _swig_new_instance_method(_operators.TimeDependentOperator_SetTime)

    def isExplicit(self):
        r"""isExplicit(TimeDependentOperator self) -> bool"""
        return _operators.TimeDependentOperator_isExplicit(self)
    isExplicit = _swig_new_instance_method(_operators.TimeDependentOperator_isExplicit)

    def isImplicit(self):
        r"""isImplicit(TimeDependentOperator self) -> bool"""
        return _operators.TimeDependentOperator_isImplicit(self)
    isImplicit = _swig_new_instance_method(_operators.TimeDependentOperator_isImplicit)

    def isHomogeneous(self):
        r"""isHomogeneous(TimeDependentOperator self) -> bool"""
        return _operators.TimeDependentOperator_isHomogeneous(self)
    isHomogeneous = _swig_new_instance_method(_operators.TimeDependentOperator_isHomogeneous)

    def GetEvalMode(self):
        r"""GetEvalMode(TimeDependentOperator self) -> mfem::TimeDependentOperator::EvalMode"""
        return _operators.TimeDependentOperator_GetEvalMode(self)
    GetEvalMode = _swig_new_instance_method(_operators.TimeDependentOperator_GetEvalMode)

    def SetEvalMode(self, new_eval_mode):
        r"""SetEvalMode(TimeDependentOperator self, mfem::TimeDependentOperator::EvalMode const new_eval_mode)"""
        return _operators.TimeDependentOperator_SetEvalMode(self, new_eval_mode)
    SetEvalMode = _swig_new_instance_method(_operators.TimeDependentOperator_SetEvalMode)

    def ExplicitMult(self, x, y):
        r"""ExplicitMult(TimeDependentOperator self, Vector x, Vector y)"""
        return _operators.TimeDependentOperator_ExplicitMult(self, x, y)
    ExplicitMult = _swig_new_instance_method(_operators.TimeDependentOperator_ExplicitMult)

    def ImplicitMult(self, x, k, y):
        r"""ImplicitMult(TimeDependentOperator self, Vector x, Vector k, Vector y)"""
        return _operators.TimeDependentOperator_ImplicitMult(self, x, k, y)
    ImplicitMult = _swig_new_instance_method(_operators.TimeDependentOperator_ImplicitMult)

    def Mult(self, x, y):
        r"""Mult(TimeDependentOperator self, Vector x, Vector y)"""
        return _operators.TimeDependentOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.TimeDependentOperator_Mult)

    def ImplicitSolve(self, dt, x, k):
        r"""ImplicitSolve(TimeDependentOperator self, double const dt, Vector x, Vector k)"""
        return _operators.TimeDependentOperator_ImplicitSolve(self, dt, x, k)
    ImplicitSolve = _swig_new_instance_method(_operators.TimeDependentOperator_ImplicitSolve)

    def GetImplicitGradient(self, x, k, shift):
        r"""GetImplicitGradient(TimeDependentOperator self, Vector x, Vector k, double shift) -> Operator"""
        return _operators.TimeDependentOperator_GetImplicitGradient(self, x, k, shift)
    GetImplicitGradient = _swig_new_instance_method(_operators.TimeDependentOperator_GetImplicitGradient)

    def GetExplicitGradient(self, x):
        r"""GetExplicitGradient(TimeDependentOperator self, Vector x) -> Operator"""
        return _operators.TimeDependentOperator_GetExplicitGradient(self, x)
    GetExplicitGradient = _swig_new_instance_method(_operators.TimeDependentOperator_GetExplicitGradient)

    def SUNImplicitSetup(self, x, fx, jok, jcur, gamma):
        r"""SUNImplicitSetup(TimeDependentOperator self, Vector x, Vector fx, int jok, int * jcur, double gamma) -> int"""
        return _operators.TimeDependentOperator_SUNImplicitSetup(self, x, fx, jok, jcur, gamma)
    SUNImplicitSetup = _swig_new_instance_method(_operators.TimeDependentOperator_SUNImplicitSetup)

    def SUNImplicitSolve(self, b, x, tol):
        r"""SUNImplicitSolve(TimeDependentOperator self, Vector b, Vector x, double tol) -> int"""
        return _operators.TimeDependentOperator_SUNImplicitSolve(self, b, x, tol)
    SUNImplicitSolve = _swig_new_instance_method(_operators.TimeDependentOperator_SUNImplicitSolve)

    def SUNMassSetup(self):
        r"""SUNMassSetup(TimeDependentOperator self) -> int"""
        return _operators.TimeDependentOperator_SUNMassSetup(self)
    SUNMassSetup = _swig_new_instance_method(_operators.TimeDependentOperator_SUNMassSetup)

    def SUNMassSolve(self, b, x, tol):
        r"""SUNMassSolve(TimeDependentOperator self, Vector b, Vector x, double tol) -> int"""
        return _operators.TimeDependentOperator_SUNMassSolve(self, b, x, tol)
    SUNMassSolve = _swig_new_instance_method(_operators.TimeDependentOperator_SUNMassSolve)

    def SUNMassMult(self, x, v):
        r"""SUNMassMult(TimeDependentOperator self, Vector x, Vector v) -> int"""
        return _operators.TimeDependentOperator_SUNMassMult(self, x, v)
    SUNMassMult = _swig_new_instance_method(_operators.TimeDependentOperator_SUNMassMult)
    __swig_destroy__ = _operators.delete_TimeDependentOperator
    def __disown__(self):
        self.this.disown()
        _operators.disown_TimeDependentOperator(self)
        return weakref.proxy(self)

# Register TimeDependentOperator in _operators:
_operators.TimeDependentOperator_swigregister(TimeDependentOperator)

class TimeDependentAdjointOperator(TimeDependentOperator):
    r"""Proxy of C++ mfem::TimeDependentAdjointOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _operators.delete_TimeDependentAdjointOperator

    def QuadratureIntegration(self, y, qdot):
        r"""QuadratureIntegration(TimeDependentAdjointOperator self, Vector y, Vector qdot)"""
        return _operators.TimeDependentAdjointOperator_QuadratureIntegration(self, y, qdot)
    QuadratureIntegration = _swig_new_instance_method(_operators.TimeDependentAdjointOperator_QuadratureIntegration)

    def AdjointRateMult(self, y, yB, yBdot):
        r"""AdjointRateMult(TimeDependentAdjointOperator self, Vector y, Vector yB, Vector yBdot)"""
        return _operators.TimeDependentAdjointOperator_AdjointRateMult(self, y, yB, yBdot)
    AdjointRateMult = _swig_new_instance_method(_operators.TimeDependentAdjointOperator_AdjointRateMult)

    def QuadratureSensitivityMult(self, y, yB, qBdot):
        r"""QuadratureSensitivityMult(TimeDependentAdjointOperator self, Vector y, Vector yB, Vector qBdot)"""
        return _operators.TimeDependentAdjointOperator_QuadratureSensitivityMult(self, y, yB, qBdot)
    QuadratureSensitivityMult = _swig_new_instance_method(_operators.TimeDependentAdjointOperator_QuadratureSensitivityMult)

    def SUNImplicitSetupB(self, t, x, xB, fxB, jokB, jcurB, gammaB):
        r"""SUNImplicitSetupB(TimeDependentAdjointOperator self, double const t, Vector x, Vector xB, Vector fxB, int jokB, int * jcurB, double gammaB) -> int"""
        return _operators.TimeDependentAdjointOperator_SUNImplicitSetupB(self, t, x, xB, fxB, jokB, jcurB, gammaB)
    SUNImplicitSetupB = _swig_new_instance_method(_operators.TimeDependentAdjointOperator_SUNImplicitSetupB)

    def SUNImplicitSolveB(self, x, b, tol):
        r"""SUNImplicitSolveB(TimeDependentAdjointOperator self, Vector x, Vector b, double tol) -> int"""
        return _operators.TimeDependentAdjointOperator_SUNImplicitSolveB(self, x, b, tol)
    SUNImplicitSolveB = _swig_new_instance_method(_operators.TimeDependentAdjointOperator_SUNImplicitSolveB)

    def GetAdjointHeight(self):
        r"""GetAdjointHeight(TimeDependentAdjointOperator self) -> int"""
        return _operators.TimeDependentAdjointOperator_GetAdjointHeight(self)
    GetAdjointHeight = _swig_new_instance_method(_operators.TimeDependentAdjointOperator_GetAdjointHeight)

# Register TimeDependentAdjointOperator in _operators:
_operators.TimeDependentAdjointOperator_swigregister(TimeDependentAdjointOperator)

class SecondOrderTimeDependentOperator(TimeDependentOperator):
    r"""Proxy of C++ mfem::SecondOrderTimeDependentOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(SecondOrderTimeDependentOperator self, int n=0, double t_=0.0, mfem::TimeDependentOperator::Type type_=EXPLICIT) -> SecondOrderTimeDependentOperator
        __init__(SecondOrderTimeDependentOperator self, int h, int w, double t_=0.0, mfem::TimeDependentOperator::Type type_=EXPLICIT) -> SecondOrderTimeDependentOperator
        """
        _operators.SecondOrderTimeDependentOperator_swiginit(self, _operators.new_SecondOrderTimeDependentOperator(*args))

    def Mult(self, *args):
        r"""
        Mult(SecondOrderTimeDependentOperator self, Vector x, Vector y)
        Mult(SecondOrderTimeDependentOperator self, Vector x, Vector dxdt, Vector y)
        """
        return _operators.SecondOrderTimeDependentOperator_Mult(self, *args)
    Mult = _swig_new_instance_method(_operators.SecondOrderTimeDependentOperator_Mult)

    def ImplicitSolve(self, *args):
        r"""
        ImplicitSolve(SecondOrderTimeDependentOperator self, double const dt, Vector x, Vector k)
        ImplicitSolve(SecondOrderTimeDependentOperator self, double const dt0, double const dt1, Vector x, Vector dxdt, Vector k)
        """
        return _operators.SecondOrderTimeDependentOperator_ImplicitSolve(self, *args)
    ImplicitSolve = _swig_new_instance_method(_operators.SecondOrderTimeDependentOperator_ImplicitSolve)
    __swig_destroy__ = _operators.delete_SecondOrderTimeDependentOperator

# Register SecondOrderTimeDependentOperator in _operators:
_operators.SecondOrderTimeDependentOperator_swigregister(SecondOrderTimeDependentOperator)

class Solver(Operator):
    r"""Proxy of C++ mfem::Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    iterative_mode = property(_operators.Solver_iterative_mode_get, _operators.Solver_iterative_mode_set, doc=r"""iterative_mode : bool""")

    def __init__(self, *args):
        r"""
        __init__(Solver self, int s=0, bool iter_mode=False) -> Solver
        __init__(Solver self, int h, int w, bool iter_mode=False) -> Solver
        """
        if self.__class__ == Solver:
            _self = None
        else:
            _self = self
        _operators.Solver_swiginit(self, _operators.new_Solver(_self, *args))

    def SetOperator(self, op):
        r"""SetOperator(Solver self, Operator op)"""
        return _operators.Solver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_operators.Solver_SetOperator)
    __swig_destroy__ = _operators.delete_Solver
    def __disown__(self):
        self.this.disown()
        _operators.disown_Solver(self)
        return weakref.proxy(self)

# Register Solver in _operators:
_operators.Solver_swigregister(Solver)

class IdentityOperator(Operator):
    r"""Proxy of C++ mfem::IdentityOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, n):
        r"""__init__(IdentityOperator self, int n) -> IdentityOperator"""
        _operators.IdentityOperator_swiginit(self, _operators.new_IdentityOperator(n))

    def Mult(self, x, y):
        r"""Mult(IdentityOperator self, Vector x, Vector y)"""
        return _operators.IdentityOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.IdentityOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(IdentityOperator self, Vector x, Vector y)"""
        return _operators.IdentityOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_operators.IdentityOperator_MultTranspose)
    __swig_destroy__ = _operators.delete_IdentityOperator

# Register IdentityOperator in _operators:
_operators.IdentityOperator_swigregister(IdentityOperator)


def IsIdentityProlongation(P):
    r"""IsIdentityProlongation(Operator P) -> bool"""
    return _operators.IsIdentityProlongation(P)
IsIdentityProlongation = _operators.IsIdentityProlongation
class ScaledOperator(Operator):
    r"""Proxy of C++ mfem::ScaledOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, A, a):
        r"""__init__(ScaledOperator self, Operator A, double a) -> ScaledOperator"""
        _operators.ScaledOperator_swiginit(self, _operators.new_ScaledOperator(A, a))

    def Mult(self, x, y):
        r"""Mult(ScaledOperator self, Vector x, Vector y)"""
        return _operators.ScaledOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.ScaledOperator_Mult)
    __swig_destroy__ = _operators.delete_ScaledOperator

# Register ScaledOperator in _operators:
_operators.ScaledOperator_swigregister(ScaledOperator)

class TransposeOperator(Operator):
    r"""Proxy of C++ mfem::TransposeOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(TransposeOperator self, Operator a) -> TransposeOperator
        __init__(TransposeOperator self, Operator a) -> TransposeOperator
        """
        _operators.TransposeOperator_swiginit(self, _operators.new_TransposeOperator(*args))

    def Mult(self, x, y):
        r"""Mult(TransposeOperator self, Vector x, Vector y)"""
        return _operators.TransposeOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.TransposeOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(TransposeOperator self, Vector x, Vector y)"""
        return _operators.TransposeOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_operators.TransposeOperator_MultTranspose)
    __swig_destroy__ = _operators.delete_TransposeOperator

# Register TransposeOperator in _operators:
_operators.TransposeOperator_swigregister(TransposeOperator)

class ProductOperator(Operator):
    r"""Proxy of C++ mfem::ProductOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, A, B, ownA, ownB):
        r"""__init__(ProductOperator self, Operator A, Operator B, bool ownA, bool ownB) -> ProductOperator"""
        _operators.ProductOperator_swiginit(self, _operators.new_ProductOperator(A, B, ownA, ownB))

    def Mult(self, x, y):
        r"""Mult(ProductOperator self, Vector x, Vector y)"""
        return _operators.ProductOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.ProductOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(ProductOperator self, Vector x, Vector y)"""
        return _operators.ProductOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_operators.ProductOperator_MultTranspose)
    __swig_destroy__ = _operators.delete_ProductOperator

# Register ProductOperator in _operators:
_operators.ProductOperator_swigregister(ProductOperator)

class RAPOperator(Operator):
    r"""Proxy of C++ mfem::RAPOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, Rt_, A_, P_):
        r"""__init__(RAPOperator self, Operator Rt_, Operator A_, Operator P_) -> RAPOperator"""
        _operators.RAPOperator_swiginit(self, _operators.new_RAPOperator(Rt_, A_, P_))

    def GetMemoryClass(self):
        r"""GetMemoryClass(RAPOperator self) -> mfem::MemoryClass"""
        return _operators.RAPOperator_GetMemoryClass(self)
    GetMemoryClass = _swig_new_instance_method(_operators.RAPOperator_GetMemoryClass)

    def Mult(self, x, y):
        r"""Mult(RAPOperator self, Vector x, Vector y)"""
        return _operators.RAPOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.RAPOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(RAPOperator self, Vector x, Vector y)"""
        return _operators.RAPOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_operators.RAPOperator_MultTranspose)
    __swig_destroy__ = _operators.delete_RAPOperator

# Register RAPOperator in _operators:
_operators.RAPOperator_swigregister(RAPOperator)

class TripleProductOperator(Operator):
    r"""Proxy of C++ mfem::TripleProductOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, A, B, C, ownA, ownB, ownC):
        r"""__init__(TripleProductOperator self, Operator A, Operator B, Operator C, bool ownA, bool ownB, bool ownC) -> TripleProductOperator"""
        _operators.TripleProductOperator_swiginit(self, _operators.new_TripleProductOperator(A, B, C, ownA, ownB, ownC))

    def GetMemoryClass(self):
        r"""GetMemoryClass(TripleProductOperator self) -> mfem::MemoryClass"""
        return _operators.TripleProductOperator_GetMemoryClass(self)
    GetMemoryClass = _swig_new_instance_method(_operators.TripleProductOperator_GetMemoryClass)

    def Mult(self, x, y):
        r"""Mult(TripleProductOperator self, Vector x, Vector y)"""
        return _operators.TripleProductOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.TripleProductOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(TripleProductOperator self, Vector x, Vector y)"""
        return _operators.TripleProductOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_operators.TripleProductOperator_MultTranspose)
    __swig_destroy__ = _operators.delete_TripleProductOperator

# Register TripleProductOperator in _operators:
_operators.TripleProductOperator_swigregister(TripleProductOperator)

class ConstrainedOperator(Operator):
    r"""Proxy of C++ mfem::ConstrainedOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args, **kwargs):
        r"""__init__(ConstrainedOperator self, Operator A, intArray list, bool own_A=False, mfem::Operator::DiagonalPolicy diag_policy=DIAG_ONE) -> ConstrainedOperator"""
        _operators.ConstrainedOperator_swiginit(self, _operators.new_ConstrainedOperator(*args, **kwargs))

    def GetMemoryClass(self):
        r"""GetMemoryClass(ConstrainedOperator self) -> mfem::MemoryClass"""
        return _operators.ConstrainedOperator_GetMemoryClass(self)
    GetMemoryClass = _swig_new_instance_method(_operators.ConstrainedOperator_GetMemoryClass)

    def SetDiagonalPolicy(self, _diag_policy):
        r"""SetDiagonalPolicy(ConstrainedOperator self, mfem::Operator::DiagonalPolicy const _diag_policy)"""
        return _operators.ConstrainedOperator_SetDiagonalPolicy(self, _diag_policy)
    SetDiagonalPolicy = _swig_new_instance_method(_operators.ConstrainedOperator_SetDiagonalPolicy)

    def EliminateRHS(self, x, b):
        r"""EliminateRHS(ConstrainedOperator self, Vector x, Vector b)"""
        return _operators.ConstrainedOperator_EliminateRHS(self, x, b)
    EliminateRHS = _swig_new_instance_method(_operators.ConstrainedOperator_EliminateRHS)

    def Mult(self, x, y):
        r"""Mult(ConstrainedOperator self, Vector x, Vector y)"""
        return _operators.ConstrainedOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.ConstrainedOperator_Mult)
    __swig_destroy__ = _operators.delete_ConstrainedOperator

# Register ConstrainedOperator in _operators:
_operators.ConstrainedOperator_swigregister(ConstrainedOperator)

class RectangularConstrainedOperator(Operator):
    r"""Proxy of C++ mfem::RectangularConstrainedOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, A, trial_list, test_list, own_A=False):
        r"""__init__(RectangularConstrainedOperator self, Operator A, intArray trial_list, intArray test_list, bool own_A=False) -> RectangularConstrainedOperator"""
        _operators.RectangularConstrainedOperator_swiginit(self, _operators.new_RectangularConstrainedOperator(A, trial_list, test_list, own_A))

    def GetMemoryClass(self):
        r"""GetMemoryClass(RectangularConstrainedOperator self) -> mfem::MemoryClass"""
        return _operators.RectangularConstrainedOperator_GetMemoryClass(self)
    GetMemoryClass = _swig_new_instance_method(_operators.RectangularConstrainedOperator_GetMemoryClass)

    def EliminateRHS(self, x, b):
        r"""EliminateRHS(RectangularConstrainedOperator self, Vector x, Vector b)"""
        return _operators.RectangularConstrainedOperator_EliminateRHS(self, x, b)
    EliminateRHS = _swig_new_instance_method(_operators.RectangularConstrainedOperator_EliminateRHS)

    def Mult(self, x, y):
        r"""Mult(RectangularConstrainedOperator self, Vector x, Vector y)"""
        return _operators.RectangularConstrainedOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.RectangularConstrainedOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(RectangularConstrainedOperator self, Vector x, Vector y)"""
        return _operators.RectangularConstrainedOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_operators.RectangularConstrainedOperator_MultTranspose)
    __swig_destroy__ = _operators.delete_RectangularConstrainedOperator

# Register RectangularConstrainedOperator in _operators:
_operators.RectangularConstrainedOperator_swigregister(RectangularConstrainedOperator)

class PowerMethod(object):
    r"""Proxy of C++ mfem::PowerMethod class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(PowerMethod self) -> PowerMethod"""
        _operators.PowerMethod_swiginit(self, _operators.new_PowerMethod())

    def EstimateLargestEigenvalue(self, opr, v0, numSteps=10, tolerance=1e-8, seed=12345):
        r"""EstimateLargestEigenvalue(PowerMethod self, Operator opr, Vector v0, int numSteps=10, double tolerance=1e-8, int seed=12345) -> double"""
        return _operators.PowerMethod_EstimateLargestEigenvalue(self, opr, v0, numSteps, tolerance, seed)
    EstimateLargestEigenvalue = _swig_new_instance_method(_operators.PowerMethod_EstimateLargestEigenvalue)
    __swig_destroy__ = _operators.delete_PowerMethod

# Register PowerMethod in _operators:
_operators.PowerMethod_swigregister(PowerMethod)

class PyOperatorBase(Operator):
    r"""Proxy of C++ mfem::PyOperatorBase class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(PyOperatorBase self, int s=0) -> PyOperatorBase
        __init__(PyOperatorBase self, int h, int w) -> PyOperatorBase
        """
        if self.__class__ == PyOperatorBase:
            _self = None
        else:
            _self = self
        _operators.PyOperatorBase_swiginit(self, _operators.new_PyOperatorBase(_self, *args))

    def Mult(self, x, y):
        r"""Mult(PyOperatorBase self, Vector x, Vector y)"""
        return _operators.PyOperatorBase_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.PyOperatorBase_Mult)

    def _EvalMult(self, arg0):
        r"""_EvalMult(PyOperatorBase self, Vector arg0) -> Vector"""
        return _operators.PyOperatorBase__EvalMult(self, arg0)
    _EvalMult = _swig_new_instance_method(_operators.PyOperatorBase__EvalMult)
    __swig_destroy__ = _operators.delete_PyOperatorBase
    def __disown__(self):
        self.this.disown()
        _operators.disown_PyOperatorBase(self)
        return weakref.proxy(self)

# Register PyOperatorBase in _operators:
_operators.PyOperatorBase_swigregister(PyOperatorBase)

class PyTimeDependentOperatorBase(TimeDependentOperator):
    r"""Proxy of C++ mfem::PyTimeDependentOperatorBase class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(PyTimeDependentOperatorBase self, int n=0, double _t=0.0) -> PyTimeDependentOperatorBase
        __init__(PyTimeDependentOperatorBase self, int h, int w, double _t=0.0) -> PyTimeDependentOperatorBase
        """
        if self.__class__ == PyTimeDependentOperatorBase:
            _self = None
        else:
            _self = self
        _operators.PyTimeDependentOperatorBase_swiginit(self, _operators.new_PyTimeDependentOperatorBase(_self, *args))

    def Mult(self, x, y):
        r"""Mult(PyTimeDependentOperatorBase self, Vector x, Vector y)"""
        return _operators.PyTimeDependentOperatorBase_Mult(self, x, y)
    Mult = _swig_new_instance_method(_operators.PyTimeDependentOperatorBase_Mult)

    def _EvalMult(self, arg0):
        r"""_EvalMult(PyTimeDependentOperatorBase self, Vector arg0) -> Vector"""
        return _operators.PyTimeDependentOperatorBase__EvalMult(self, arg0)
    _EvalMult = _swig_new_instance_method(_operators.PyTimeDependentOperatorBase__EvalMult)
    __swig_destroy__ = _operators.delete_PyTimeDependentOperatorBase
    def __disown__(self):
        self.this.disown()
        _operators.disown_PyTimeDependentOperatorBase(self)
        return weakref.proxy(self)

# Register PyTimeDependentOperatorBase in _operators:
_operators.PyTimeDependentOperatorBase_swigregister(PyTimeDependentOperatorBase)


class PyOperator(PyOperatorBase):
   def __init__(self, *args):
       PyOperatorBase.__init__(self, *args)
   def _EvalMult(self, x):
       return self.EvalMult(x.GetDataArray())
   def EvalMult(self, x):
       raise NotImplementedError('you must specify this method')

class PyTimeDependentOperator(PyTimeDependentOperatorBase):
   def __init__(self, *args):  
       PyTimeDependentOperatorBase.__init__(self, *args)
   def _EvalMult(self, x):
       return self.EvalMult(x.GetDataArray())
   def EvalMult(self, x):
       raise NotImplementedError('you must specify this method')







# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _pmesh
else:
    import _pmesh

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _pmesh.SWIG_PyInstanceMethod_New
_swig_new_static_method = _pmesh.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _pmesh.MFEM_VERSION

MFEM_VERSION_STRING = _pmesh.MFEM_VERSION_STRING

MFEM_VERSION_TYPE = _pmesh.MFEM_VERSION_TYPE

MFEM_VERSION_TYPE_RELEASE = _pmesh.MFEM_VERSION_TYPE_RELEASE

MFEM_VERSION_TYPE_DEVELOPMENT = _pmesh.MFEM_VERSION_TYPE_DEVELOPMENT

MFEM_VERSION_MAJOR = _pmesh.MFEM_VERSION_MAJOR

MFEM_VERSION_MINOR = _pmesh.MFEM_VERSION_MINOR

MFEM_VERSION_PATCH = _pmesh.MFEM_VERSION_PATCH

MFEM_HYPRE_VERSION = _pmesh.MFEM_HYPRE_VERSION

import mfem._par.mesh
import mfem._par.matrix
import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.globals
import mfem._par.densemat
import mfem._par.geom
import mfem._par.intrules
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.coefficient
import mfem._par.sparsemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.fespace
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.bilininteg
import mfem._par.linearform
import mfem._par.pncmesh
import mfem._par.communication
import mfem._par.sets
class ParMesh(mfem._par.mesh.Mesh):
    r"""Proxy of C++ mfem::ParMesh class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Finalize(self, refine=False, fix_orientation=False):
        r"""Finalize(ParMesh self, bool refine=False, bool fix_orientation=False)"""
        return _pmesh.ParMesh_Finalize(self, refine, fix_orientation)
    Finalize = _swig_new_instance_method(_pmesh.ParMesh_Finalize)

    def SetAttributes(self):
        r"""SetAttributes(ParMesh self)"""
        return _pmesh.ParMesh_SetAttributes(self)
    SetAttributes = _swig_new_instance_method(_pmesh.ParMesh_SetAttributes)

    def GetComm(self):
        r"""GetComm(ParMesh self) -> MPI_Comm"""
        return _pmesh.ParMesh_GetComm(self)
    GetComm = _swig_new_instance_method(_pmesh.ParMesh_GetComm)

    def GetNRanks(self):
        r"""GetNRanks(ParMesh self) -> int"""
        return _pmesh.ParMesh_GetNRanks(self)
    GetNRanks = _swig_new_instance_method(_pmesh.ParMesh_GetNRanks)

    def GetMyRank(self):
        r"""GetMyRank(ParMesh self) -> int"""
        return _pmesh.ParMesh_GetMyRank(self)
    GetMyRank = _swig_new_instance_method(_pmesh.ParMesh_GetMyRank)

    def GetLocalElementNum(self, global_element_num):
        r"""GetLocalElementNum(ParMesh self, long global_element_num) -> int"""
        return _pmesh.ParMesh_GetLocalElementNum(self, global_element_num)
    GetLocalElementNum = _swig_new_instance_method(_pmesh.ParMesh_GetLocalElementNum)

    def GetGlobalElementNum(self, local_element_num):
        r"""GetGlobalElementNum(ParMesh self, int local_element_num) -> long"""
        return _pmesh.ParMesh_GetGlobalElementNum(self, local_element_num)
    GetGlobalElementNum = _swig_new_instance_method(_pmesh.ParMesh_GetGlobalElementNum)
    gtopo = property(_pmesh.ParMesh_gtopo_get, doc=r"""gtopo : mfem::GroupTopology""")
    have_face_nbr_data = property(_pmesh.ParMesh_have_face_nbr_data_get, _pmesh.ParMesh_have_face_nbr_data_set, doc=r"""have_face_nbr_data : bool""")
    face_nbr_group = property(_pmesh.ParMesh_face_nbr_group_get, _pmesh.ParMesh_face_nbr_group_set, doc=r"""face_nbr_group : mfem::Array<(int)>""")
    face_nbr_elements_offset = property(_pmesh.ParMesh_face_nbr_elements_offset_get, _pmesh.ParMesh_face_nbr_elements_offset_set, doc=r"""face_nbr_elements_offset : mfem::Array<(int)>""")
    face_nbr_vertices_offset = property(_pmesh.ParMesh_face_nbr_vertices_offset_get, _pmesh.ParMesh_face_nbr_vertices_offset_set, doc=r"""face_nbr_vertices_offset : mfem::Array<(int)>""")
    face_nbr_elements = property(_pmesh.ParMesh_face_nbr_elements_get, doc=r"""face_nbr_elements : mfem::Array<(p.mfem::Element)>""")
    face_nbr_vertices = property(_pmesh.ParMesh_face_nbr_vertices_get, doc=r"""face_nbr_vertices : mfem::Array<(mfem::Vertex)>""")
    send_face_nbr_elements = property(_pmesh.ParMesh_send_face_nbr_elements_get, _pmesh.ParMesh_send_face_nbr_elements_set, doc=r"""send_face_nbr_elements : mfem::Table""")
    send_face_nbr_vertices = property(_pmesh.ParMesh_send_face_nbr_vertices_get, _pmesh.ParMesh_send_face_nbr_vertices_set, doc=r"""send_face_nbr_vertices : mfem::Table""")
    pncmesh = property(_pmesh.ParMesh_pncmesh_get, _pmesh.ParMesh_pncmesh_set, doc=r"""pncmesh : p.mfem::ParNCMesh""")

    def GetNGroups(self):
        r"""GetNGroups(ParMesh self) -> int"""
        return _pmesh.ParMesh_GetNGroups(self)
    GetNGroups = _swig_new_instance_method(_pmesh.ParMesh_GetNGroups)

    def GroupNVertices(self, group):
        r"""GroupNVertices(ParMesh self, int group) -> int"""
        return _pmesh.ParMesh_GroupNVertices(self, group)
    GroupNVertices = _swig_new_instance_method(_pmesh.ParMesh_GroupNVertices)

    def GroupNEdges(self, group):
        r"""GroupNEdges(ParMesh self, int group) -> int"""
        return _pmesh.ParMesh_GroupNEdges(self, group)
    GroupNEdges = _swig_new_instance_method(_pmesh.ParMesh_GroupNEdges)

    def GroupNTriangles(self, group):
        r"""GroupNTriangles(ParMesh self, int group) -> int"""
        return _pmesh.ParMesh_GroupNTriangles(self, group)
    GroupNTriangles = _swig_new_instance_method(_pmesh.ParMesh_GroupNTriangles)

    def GroupNQuadrilaterals(self, group):
        r"""GroupNQuadrilaterals(ParMesh self, int group) -> int"""
        return _pmesh.ParMesh_GroupNQuadrilaterals(self, group)
    GroupNQuadrilaterals = _swig_new_instance_method(_pmesh.ParMesh_GroupNQuadrilaterals)

    def GroupVertex(self, group, i):
        r"""GroupVertex(ParMesh self, int group, int i) -> int"""
        return _pmesh.ParMesh_GroupVertex(self, group, i)
    GroupVertex = _swig_new_instance_method(_pmesh.ParMesh_GroupVertex)

    def GroupEdge(self, group, i, *args):
        if len(args) == 0:
            from mfem.par import intp  
            edge = intp()
            o = intp()  
            _pmesh.ParMesh_GroupEdge(self, group, i, edge, o)
            return edge.value(), o.value()
        else:
            return _pmesh.ParMesh_GroupEdge(self, group, i, *args)      



    def GroupTriangle(self, group, i, face, o):
        r"""GroupTriangle(ParMesh self, int group, int i, int & face, int & o)"""
        return _pmesh.ParMesh_GroupTriangle(self, group, i, face, o)
    GroupTriangle = _swig_new_instance_method(_pmesh.ParMesh_GroupTriangle)

    def GroupQuadrilateral(self, group, i, face, o):
        r"""GroupQuadrilateral(ParMesh self, int group, int i, int & face, int & o)"""
        return _pmesh.ParMesh_GroupQuadrilateral(self, group, i, face, o)
    GroupQuadrilateral = _swig_new_instance_method(_pmesh.ParMesh_GroupQuadrilateral)

    def GenerateOffsets(self, N, loc_sizes, offsets):
        r"""GenerateOffsets(ParMesh self, int N, HYPRE_Int [] loc_sizes, mfem::Array< HYPRE_Int > *[] offsets)"""
        return _pmesh.ParMesh_GenerateOffsets(self, N, loc_sizes, offsets)
    GenerateOffsets = _swig_new_instance_method(_pmesh.ParMesh_GenerateOffsets)

    def ExchangeFaceNbrData(self):
        r"""ExchangeFaceNbrData(ParMesh self)"""
        return _pmesh.ParMesh_ExchangeFaceNbrData(self)
    ExchangeFaceNbrData = _swig_new_instance_method(_pmesh.ParMesh_ExchangeFaceNbrData)

    def ExchangeFaceNbrNodes(self):
        r"""ExchangeFaceNbrNodes(ParMesh self)"""
        return _pmesh.ParMesh_ExchangeFaceNbrNodes(self)
    ExchangeFaceNbrNodes = _swig_new_instance_method(_pmesh.ParMesh_ExchangeFaceNbrNodes)

    def SetCurvature(self, order, discont=False, space_dim=-1, ordering=1):
        r"""SetCurvature(ParMesh self, int order, bool discont=False, int space_dim=-1, int ordering=1)"""
        return _pmesh.ParMesh_SetCurvature(self, order, discont, space_dim, ordering)
    SetCurvature = _swig_new_instance_method(_pmesh.ParMesh_SetCurvature)

    def GetNFaceNeighbors(self):
        r"""GetNFaceNeighbors(ParMesh self) -> int"""
        return _pmesh.ParMesh_GetNFaceNeighbors(self)
    GetNFaceNeighbors = _swig_new_instance_method(_pmesh.ParMesh_GetNFaceNeighbors)

    def GetFaceNbrGroup(self, fn):
        r"""GetFaceNbrGroup(ParMesh self, int fn) -> int"""
        return _pmesh.ParMesh_GetFaceNbrGroup(self, fn)
    GetFaceNbrGroup = _swig_new_instance_method(_pmesh.ParMesh_GetFaceNbrGroup)

    def GetFaceNbrRank(self, fn):
        r"""GetFaceNbrRank(ParMesh self, int fn) -> int"""
        return _pmesh.ParMesh_GetFaceNbrRank(self, fn)
    GetFaceNbrRank = _swig_new_instance_method(_pmesh.ParMesh_GetFaceNbrRank)

    def GetFaceToAllElementTable(self):
        r"""GetFaceToAllElementTable(ParMesh self) -> Table"""
        return _pmesh.ParMesh_GetFaceToAllElementTable(self)
    GetFaceToAllElementTable = _swig_new_instance_method(_pmesh.ParMesh_GetFaceToAllElementTable)

    def GetSharedFaceTransformations(self, sf, fill2=True):
        r"""GetSharedFaceTransformations(ParMesh self, int sf, bool fill2=True) -> FaceElementTransformations"""
        return _pmesh.ParMesh_GetSharedFaceTransformations(self, sf, fill2)
    GetSharedFaceTransformations = _swig_new_instance_method(_pmesh.ParMesh_GetSharedFaceTransformations)

    def GetFaceNbrElementTransformation(self, i):
        r"""GetFaceNbrElementTransformation(ParMesh self, int i) -> ElementTransformation"""
        return _pmesh.ParMesh_GetFaceNbrElementTransformation(self, i)
    GetFaceNbrElementTransformation = _swig_new_instance_method(_pmesh.ParMesh_GetFaceNbrElementTransformation)

    def GetNSharedFaces(self):
        r"""GetNSharedFaces(ParMesh self) -> int"""
        return _pmesh.ParMesh_GetNSharedFaces(self)
    GetNSharedFaces = _swig_new_instance_method(_pmesh.ParMesh_GetNSharedFaces)

    def GetSharedFace(self, sface):
        r"""GetSharedFace(ParMesh self, int sface) -> int"""
        return _pmesh.ParMesh_GetSharedFace(self, sface)
    GetSharedFace = _swig_new_instance_method(_pmesh.ParMesh_GetSharedFace)

    def ReorientTetMesh(self):
        r"""ReorientTetMesh(ParMesh self)"""
        return _pmesh.ParMesh_ReorientTetMesh(self)
    ReorientTetMesh = _swig_new_instance_method(_pmesh.ParMesh_ReorientTetMesh)

    def ReduceInt(self, value):
        r"""ReduceInt(ParMesh self, int value) -> long"""
        return _pmesh.ParMesh_ReduceInt(self, value)
    ReduceInt = _swig_new_instance_method(_pmesh.ParMesh_ReduceInt)

    def Rebalance(self, *args):
        r"""
        Rebalance(ParMesh self)
        Rebalance(ParMesh self, intArray partition)
        """
        return _pmesh.ParMesh_Rebalance(self, *args)
    Rebalance = _swig_new_instance_method(_pmesh.ParMesh_Rebalance)

    def GetBoundingBox(self, ref = 2):
        from  .vector import Vector
        min = Vector()
        max = Vector()      
        _mesh.Mesh_GetBoundingBox(self, min, max, ref)      
        return min.GetDataArray().copy(), max.GetDataArray().copy()



    def GetCharacteristics(self, h_min, h_max, kappa_min, kappa_max):
        r"""GetCharacteristics(ParMesh self, double & h_min, double & h_max, double & kappa_min, double & kappa_max)"""
        return _pmesh.ParMesh_GetCharacteristics(self, h_min, h_max, kappa_min, kappa_max)
    GetCharacteristics = _swig_new_instance_method(_pmesh.ParMesh_GetCharacteristics)

    def PrintVTU(self, *args, **kwargs):
        r"""PrintVTU(ParMesh self, std::string pathname, mfem::VTKFormat format=ASCII, bool high_order_output=False, int compression_level=0, bool bdr=False)"""
        return _pmesh.ParMesh_PrintVTU(self, *args, **kwargs)
    PrintVTU = _swig_new_instance_method(_pmesh.ParMesh_PrintVTU)

    def FindPoints(self, pp, warn=True, inv_trans=None):            
        r"""count, element_id, integration_points = FindPoints(points, warn=True, int_trans=None)"""
        import numpy as np
        import mfem.par as mfem

        pp = np.array(pp, copy=False, dtype=float).transpose()      
        M = mfem.DenseMatrix(pp.shape[0], pp.shape[1])
        M.Assign(pp)
        elem_ids = mfem.intArray()
        int_points = mfem.IntegrationPointArray()
        count = _mesh.Mesh_FindPoints(self, M, elem_ids, int_points, warn, inv_trans)      
        elem_ids = elem_ids.ToList()
        return count, elem_ids, int_points



    def PrintSharedEntities(self, fname_prefix):
        r"""PrintSharedEntities(ParMesh self, char const * fname_prefix)"""
        return _pmesh.ParMesh_PrintSharedEntities(self, fname_prefix)
    PrintSharedEntities = _swig_new_instance_method(_pmesh.ParMesh_PrintSharedEntities)
    __swig_destroy__ = _pmesh.delete_ParMesh

    def __init__(self, *args):
        r"""
        __init__(ParMesh self, ParMesh pmesh, bool copy_nodes=True) -> ParMesh
        __init__(ParMesh self, MPI_Comm comm, Mesh mesh, int * partitioning_=None, int part_method=1) -> ParMesh
        __init__(ParMesh self, MPI_Comm comm, std::istream & input, bool refine=True) -> ParMesh
        __init__(ParMesh self, ParMesh orig_mesh, int ref_factor, int ref_type) -> ParMesh
        __init__(ParMesh self, MPI_Comm comm, char const * mesh_file) -> ParMesh
        """
        _pmesh.ParMesh_swiginit(self, _pmesh.new_ParMesh(*args))

    def ParPrintToFile(self, mesh_file, precision):
        r"""ParPrintToFile(ParMesh self, char const * mesh_file, int const precision)"""
        return _pmesh.ParMesh_ParPrintToFile(self, mesh_file, precision)
    ParPrintToFile = _swig_new_instance_method(_pmesh.ParMesh_ParPrintToFile)

    def Print(self, *args):
        r"""
        Print(ParMesh self, std::ostream & out=out)
        Print(ParMesh self, char const * file, int precision=8)
        """
        return _pmesh.ParMesh_Print(self, *args)
    Print = _swig_new_instance_method(_pmesh.ParMesh_Print)

    def PrintXG(self, *args):
        r"""
        PrintXG(ParMesh self, std::ostream & out=out)
        PrintXG(ParMesh self, char const * file, int precision=8)
        """
        return _pmesh.ParMesh_PrintXG(self, *args)
    PrintXG = _swig_new_instance_method(_pmesh.ParMesh_PrintXG)

    def PrintAsOne(self, *args):
        r"""
        PrintAsOne(ParMesh self, std::ostream & out=out)
        PrintAsOne(ParMesh self, char const * file, int precision=8)
        """
        return _pmesh.ParMesh_PrintAsOne(self, *args)
    PrintAsOne = _swig_new_instance_method(_pmesh.ParMesh_PrintAsOne)

    def PrintAsOneXG(self, *args):
        r"""
        PrintAsOneXG(ParMesh self, std::ostream & out=out)
        PrintAsOneXG(ParMesh self, char const * file, int precision=8)
        """
        return _pmesh.ParMesh_PrintAsOneXG(self, *args)
    PrintAsOneXG = _swig_new_instance_method(_pmesh.ParMesh_PrintAsOneXG)

    def PrintInfo(self, *args):
        r"""
        PrintInfo(ParMesh self, std::ostream & out=out)
        PrintInfo(ParMesh self, char const * file, int precision=8)
        """
        return _pmesh.ParMesh_PrintInfo(self, *args)
    PrintInfo = _swig_new_instance_method(_pmesh.ParMesh_PrintInfo)

    def ParPrint(self, *args):
        r"""
        ParPrint(ParMesh self, std::ostream & out)
        ParPrint(ParMesh self, char const * file, int precision=8)
        ParPrint(ParMesh self)
        """
        return _pmesh.ParMesh_ParPrint(self, *args)
    ParPrint = _swig_new_instance_method(_pmesh.ParMesh_ParPrint)

# Register ParMesh in _pmesh:
_pmesh.ParMesh_swigregister(ParMesh)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _tetrahedron
else:
    import _tetrahedron

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _tetrahedron.SWIG_PyInstanceMethod_New
_swig_new_static_method = _tetrahedron.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._ser.fe
import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.vector
import mfem._ser.geom
import mfem._ser.intrules
import mfem._ser.densemat
import mfem._ser.operators
import mfem._ser.matrix
import mfem._ser.sparsemat
import mfem._ser.element
import mfem._ser.globals
import mfem._ser.table
import mfem._ser.hash
class Tetrahedron(mfem._ser.element.Element):
    r"""Proxy of C++ mfem::Tetrahedron class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TYPE_PU = _tetrahedron.Tetrahedron_TYPE_PU
    
    TYPE_A = _tetrahedron.Tetrahedron_TYPE_A
    
    TYPE_PF = _tetrahedron.Tetrahedron_TYPE_PF
    
    TYPE_O = _tetrahedron.Tetrahedron_TYPE_O
    
    TYPE_M = _tetrahedron.Tetrahedron_TYPE_M
    

    def __init__(self, *args):
        r"""
        __init__(Tetrahedron self) -> Tetrahedron
        __init__(Tetrahedron self, int const * ind, int attr=1) -> Tetrahedron
        __init__(Tetrahedron self, int ind1, int ind2, int ind3, int ind4, int attr=1) -> Tetrahedron
        """
        _tetrahedron.Tetrahedron_swiginit(self, _tetrahedron.new_Tetrahedron(*args))

    def Init(self, ind1, ind2, ind3, ind4, attr=1, ref_flag=0):
        r"""Init(Tetrahedron self, int ind1, int ind2, int ind3, int ind4, int attr=1, int ref_flag=0)"""
        return _tetrahedron.Tetrahedron_Init(self, ind1, ind2, ind3, ind4, attr, ref_flag)
    Init = _swig_new_instance_method(_tetrahedron.Tetrahedron_Init)

    def GetType(self):
        r"""GetType(Tetrahedron self) -> mfem::Element::Type"""
        return _tetrahedron.Tetrahedron_GetType(self)
    GetType = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetType)

    def ParseRefinementFlag(self, refinement_edges, type, flag):
        r"""ParseRefinementFlag(Tetrahedron self, int [2] refinement_edges, int & type, int & flag)"""
        return _tetrahedron.Tetrahedron_ParseRefinementFlag(self, refinement_edges, type, flag)
    ParseRefinementFlag = _swig_new_instance_method(_tetrahedron.Tetrahedron_ParseRefinementFlag)

    def CreateRefinementFlag(self, refinement_edges, type, flag=0):
        r"""CreateRefinementFlag(Tetrahedron self, int [2] refinement_edges, int type, int flag=0)"""
        return _tetrahedron.Tetrahedron_CreateRefinementFlag(self, refinement_edges, type, flag)
    CreateRefinementFlag = _swig_new_instance_method(_tetrahedron.Tetrahedron_CreateRefinementFlag)

    def GetMarkedFace(self, face, fv):
        r"""GetMarkedFace(Tetrahedron self, int const face, int * fv)"""
        return _tetrahedron.Tetrahedron_GetMarkedFace(self, face, fv)
    GetMarkedFace = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetMarkedFace)

    def GetRefinementFlag(self):
        r"""GetRefinementFlag(Tetrahedron self) -> int"""
        return _tetrahedron.Tetrahedron_GetRefinementFlag(self)
    GetRefinementFlag = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetRefinementFlag)

    def SetRefinementFlag(self, rf):
        r"""SetRefinementFlag(Tetrahedron self, int rf)"""
        return _tetrahedron.Tetrahedron_SetRefinementFlag(self, rf)
    SetRefinementFlag = _swig_new_instance_method(_tetrahedron.Tetrahedron_SetRefinementFlag)

    def NeedRefinement(self, v_to_v):
        r"""NeedRefinement(Tetrahedron self, mfem::HashTable< mfem::Hashed2 > & v_to_v) -> int"""
        return _tetrahedron.Tetrahedron_NeedRefinement(self, v_to_v)
    NeedRefinement = _swig_new_instance_method(_tetrahedron.Tetrahedron_NeedRefinement)

    def SetVertices(self, ind):
        r"""SetVertices(Tetrahedron self, int const * ind)"""
        return _tetrahedron.Tetrahedron_SetVertices(self, ind)
    SetVertices = _swig_new_instance_method(_tetrahedron.Tetrahedron_SetVertices)

    def MarkEdge(self, v_to_v, length):
        r"""MarkEdge(Tetrahedron self, DSTable v_to_v, int const * length)"""
        return _tetrahedron.Tetrahedron_MarkEdge(self, v_to_v, length)
    MarkEdge = _swig_new_instance_method(_tetrahedron.Tetrahedron_MarkEdge)

    def ResetTransform(self, tr):
        r"""ResetTransform(Tetrahedron self, int tr)"""
        return _tetrahedron.Tetrahedron_ResetTransform(self, tr)
    ResetTransform = _swig_new_instance_method(_tetrahedron.Tetrahedron_ResetTransform)

    def GetTransform(self):
        r"""GetTransform(Tetrahedron self) -> unsigned int"""
        return _tetrahedron.Tetrahedron_GetTransform(self)
    GetTransform = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetTransform)

    def PushTransform(self, tr):
        r"""PushTransform(Tetrahedron self, int tr)"""
        return _tetrahedron.Tetrahedron_PushTransform(self, tr)
    PushTransform = _swig_new_instance_method(_tetrahedron.Tetrahedron_PushTransform)

    @staticmethod
    def GetPointMatrix(transform, pm):
        r"""GetPointMatrix(unsigned int transform, DenseMatrix pm)"""
        return _tetrahedron.Tetrahedron_GetPointMatrix(transform, pm)
    GetPointMatrix = _swig_new_static_method(_tetrahedron.Tetrahedron_GetPointMatrix)

    def GetVertices(self, *args):
        r"""
        GetVertices(Tetrahedron self, intArray v)
        GetVertices(Tetrahedron self) -> int *
        """
        return _tetrahedron.Tetrahedron_GetVertices(self, *args)
    GetVertices = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetVertices)

    def GetNVertices(self):
        r"""GetNVertices(Tetrahedron self) -> int"""
        return _tetrahedron.Tetrahedron_GetNVertices(self)
    GetNVertices = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetNVertices)

    def GetNEdges(self):
        r"""GetNEdges(Tetrahedron self) -> int"""
        return _tetrahedron.Tetrahedron_GetNEdges(self)
    GetNEdges = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetNEdges)

    def GetEdgeVertices(self, ei):
        r"""GetEdgeVertices(Tetrahedron self, int ei) -> int const *"""
        return _tetrahedron.Tetrahedron_GetEdgeVertices(self, ei)
    GetEdgeVertices = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetEdgeVertices)

    def GetNFaces(self, *args):
        r"""
        GetNFaces(Tetrahedron self, int & nFaceVertices) -> int
        GetNFaces(Tetrahedron self) -> int
        """

        if len(args) == 1:
             import warnings
             warnings.warn("Tetrahedron::GetNFaces(int & nFaceVertices) is deprecated is deprecated",
         	              DeprecationWarning,)


        return _tetrahedron.Tetrahedron_GetNFaces(self, *args)


    def GetNFaceVertices(self, arg2):
        r"""GetNFaceVertices(Tetrahedron self, int arg2) -> int"""
        return _tetrahedron.Tetrahedron_GetNFaceVertices(self, arg2)
    GetNFaceVertices = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetNFaceVertices)

    def GetFaceVertices(self, fi):
        r"""GetFaceVertices(Tetrahedron self, int fi) -> int const *"""
        return _tetrahedron.Tetrahedron_GetFaceVertices(self, fi)
    GetFaceVertices = _swig_new_instance_method(_tetrahedron.Tetrahedron_GetFaceVertices)

    def Duplicate(self, m):
        r"""Duplicate(Tetrahedron self, mfem::Mesh * m) -> Element"""
        return _tetrahedron.Tetrahedron_Duplicate(self, m)
    Duplicate = _swig_new_instance_method(_tetrahedron.Tetrahedron_Duplicate)
    __swig_destroy__ = _tetrahedron.delete_Tetrahedron

# Register Tetrahedron in _tetrahedron:
_tetrahedron.Tetrahedron_swigregister(Tetrahedron)

def Tetrahedron_GetPointMatrix(transform, pm):
    r"""Tetrahedron_GetPointMatrix(unsigned int transform, DenseMatrix pm)"""
    return _tetrahedron.Tetrahedron_GetPointMatrix(transform, pm)
Tetrahedron_GetPointMatrix = _tetrahedron.Tetrahedron_GetPointMatrix


cvar = _tetrahedron.cvar




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _sets
else:
    import _sets

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _sets.SWIG_PyInstanceMethod_New
_swig_new_static_method = _sets.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.table
class IntegerSet(object):
    r"""Proxy of C++ mfem::IntegerSet class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(IntegerSet self) -> IntegerSet
        __init__(IntegerSet self, IntegerSet s) -> IntegerSet
        __init__(IntegerSet self, int const n, int const * p) -> IntegerSet
        """
        _sets.IntegerSet_swiginit(self, _sets.new_IntegerSet(*args))

    def Size(self):
        r"""Size(IntegerSet self) -> int"""
        return _sets.IntegerSet_Size(self)
    Size = _swig_new_instance_method(_sets.IntegerSet_Size)

    def PickElement(self):
        r"""PickElement(IntegerSet self) -> int"""
        return _sets.IntegerSet_PickElement(self)
    PickElement = _swig_new_instance_method(_sets.IntegerSet_PickElement)

    def PickRandomElement(self):
        r"""PickRandomElement(IntegerSet self) -> int"""
        return _sets.IntegerSet_PickRandomElement(self)
    PickRandomElement = _swig_new_instance_method(_sets.IntegerSet_PickRandomElement)

    def __eq__(self, s):
        r"""__eq__(IntegerSet self, IntegerSet s) -> int"""
        return _sets.IntegerSet___eq__(self, s)
    __eq__ = _swig_new_instance_method(_sets.IntegerSet___eq__)

    def Recreate(self, n, p):
        r"""Recreate(IntegerSet self, int const n, int const * p)"""
        return _sets.IntegerSet_Recreate(self, n, p)
    Recreate = _swig_new_instance_method(_sets.IntegerSet_Recreate)
    __swig_destroy__ = _sets.delete_IntegerSet

# Register IntegerSet in _sets:
_sets.IntegerSet_swigregister(IntegerSet)

class ListOfIntegerSets(object):
    r"""Proxy of C++ mfem::ListOfIntegerSets class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Size(self):
        r"""Size(ListOfIntegerSets self) -> int"""
        return _sets.ListOfIntegerSets_Size(self)
    Size = _swig_new_instance_method(_sets.ListOfIntegerSets_Size)

    def PickElementInSet(self, i):
        r"""PickElementInSet(ListOfIntegerSets self, int i) -> int"""
        return _sets.ListOfIntegerSets_PickElementInSet(self, i)
    PickElementInSet = _swig_new_instance_method(_sets.ListOfIntegerSets_PickElementInSet)

    def PickRandomElementInSet(self, i):
        r"""PickRandomElementInSet(ListOfIntegerSets self, int i) -> int"""
        return _sets.ListOfIntegerSets_PickRandomElementInSet(self, i)
    PickRandomElementInSet = _swig_new_instance_method(_sets.ListOfIntegerSets_PickRandomElementInSet)

    def Insert(self, s):
        r"""Insert(ListOfIntegerSets self, IntegerSet s) -> int"""
        return _sets.ListOfIntegerSets_Insert(self, s)
    Insert = _swig_new_instance_method(_sets.ListOfIntegerSets_Insert)

    def Lookup(self, s):
        r"""Lookup(ListOfIntegerSets self, IntegerSet s) -> int"""
        return _sets.ListOfIntegerSets_Lookup(self, s)
    Lookup = _swig_new_instance_method(_sets.ListOfIntegerSets_Lookup)

    def AsTable(self, t):
        r"""AsTable(ListOfIntegerSets self, Table t)"""
        return _sets.ListOfIntegerSets_AsTable(self, t)
    AsTable = _swig_new_instance_method(_sets.ListOfIntegerSets_AsTable)
    __swig_destroy__ = _sets.delete_ListOfIntegerSets

    def __init__(self):
        r"""__init__(ListOfIntegerSets self) -> ListOfIntegerSets"""
        _sets.ListOfIntegerSets_swiginit(self, _sets.new_ListOfIntegerSets())

# Register ListOfIntegerSets in _sets:
_sets.ListOfIntegerSets_swigregister(ListOfIntegerSets)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _fespace
else:
    import _fespace

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _fespace.SWIG_PyInstanceMethod_New
_swig_new_static_method = _fespace.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.array
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.operators
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
class Ordering(object):
    r"""Proxy of C++ mfem::Ordering class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    byNODES = _fespace.Ordering_byNODES
    
    byVDIM = _fespace.Ordering_byVDIM
    

    def __init__(self):
        r"""__init__(Ordering self) -> Ordering"""
        _fespace.Ordering_swiginit(self, _fespace.new_Ordering())
    __swig_destroy__ = _fespace.delete_Ordering

# Register Ordering in _fespace:
_fespace.Ordering_swigregister(Ordering)

ElementDofOrdering_NATIVE = _fespace.ElementDofOrdering_NATIVE

ElementDofOrdering_LEXICOGRAPHIC = _fespace.ElementDofOrdering_LEXICOGRAPHIC

class FiniteElementSpace(object):
    r"""Proxy of C++ mfem::FiniteElementSpace class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(FiniteElementSpace self) -> FiniteElementSpace
        __init__(FiniteElementSpace self, FiniteElementSpace orig, Mesh mesh=None, FiniteElementCollection fec=None) -> FiniteElementSpace
        __init__(FiniteElementSpace self, Mesh mesh, FiniteElementCollection fec, int vdim=1, int ordering=byNODES) -> FiniteElementSpace
        __init__(FiniteElementSpace self, Mesh mesh, mfem::NURBSExtension * ext, FiniteElementCollection fec, int vdim=1, int ordering=byNODES) -> FiniteElementSpace
        """
        _fespace.FiniteElementSpace_swiginit(self, _fespace.new_FiniteElementSpace(*args))


        '''
            FiniteElementSpace(Mesh *m, const FiniteElementCollection *f,
                          int vdim = 1, int ordering = Ordering::byNODES);
            keep reference to mesh and fec to prevent it from
            freeed from pytho garbage collector
        '''
        if hasattr(self, 'this'):
            self.mesh = args[0]
            self.fec = args[1]





    def GetMesh(self):
        r"""GetMesh(FiniteElementSpace self) -> Mesh"""
        return _fespace.FiniteElementSpace_GetMesh(self)
    GetMesh = _swig_new_instance_method(_fespace.FiniteElementSpace_GetMesh)

    def GetNURBSext(self, *args):
        r"""
        GetNURBSext(FiniteElementSpace self) -> mfem::NURBSExtension const
        GetNURBSext(FiniteElementSpace self) -> mfem::NURBSExtension *
        """
        return _fespace.FiniteElementSpace_GetNURBSext(self, *args)
    GetNURBSext = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNURBSext)

    def StealNURBSext(self):
        r"""StealNURBSext(FiniteElementSpace self) -> mfem::NURBSExtension *"""
        return _fespace.FiniteElementSpace_StealNURBSext(self)
    StealNURBSext = _swig_new_instance_method(_fespace.FiniteElementSpace_StealNURBSext)

    def Conforming(self):
        r"""Conforming(FiniteElementSpace self) -> bool"""
        return _fespace.FiniteElementSpace_Conforming(self)
    Conforming = _swig_new_instance_method(_fespace.FiniteElementSpace_Conforming)

    def Nonconforming(self):
        r"""Nonconforming(FiniteElementSpace self) -> bool"""
        return _fespace.FiniteElementSpace_Nonconforming(self)
    Nonconforming = _swig_new_instance_method(_fespace.FiniteElementSpace_Nonconforming)

    def GetConformingProlongation(self):
        r"""GetConformingProlongation(FiniteElementSpace self) -> SparseMatrix"""
        return _fespace.FiniteElementSpace_GetConformingProlongation(self)
    GetConformingProlongation = _swig_new_instance_method(_fespace.FiniteElementSpace_GetConformingProlongation)

    def GetConformingRestriction(self):
        r"""GetConformingRestriction(FiniteElementSpace self) -> SparseMatrix"""
        return _fespace.FiniteElementSpace_GetConformingRestriction(self)
    GetConformingRestriction = _swig_new_instance_method(_fespace.FiniteElementSpace_GetConformingRestriction)

    def GetProlongationMatrix(self):
        r"""GetProlongationMatrix(FiniteElementSpace self) -> Operator"""
        return _fespace.FiniteElementSpace_GetProlongationMatrix(self)
    GetProlongationMatrix = _swig_new_instance_method(_fespace.FiniteElementSpace_GetProlongationMatrix)

    def GetRestrictionMatrix(self):
        r"""GetRestrictionMatrix(FiniteElementSpace self) -> SparseMatrix"""
        return _fespace.FiniteElementSpace_GetRestrictionMatrix(self)
    GetRestrictionMatrix = _swig_new_instance_method(_fespace.FiniteElementSpace_GetRestrictionMatrix)

    def GetElementRestriction(self, e_ordering):
        r"""GetElementRestriction(FiniteElementSpace self, mfem::ElementDofOrdering e_ordering) -> Operator"""
        return _fespace.FiniteElementSpace_GetElementRestriction(self, e_ordering)
    GetElementRestriction = _swig_new_instance_method(_fespace.FiniteElementSpace_GetElementRestriction)

    def GetFaceRestriction(self, *args, **kwargs):
        r"""GetFaceRestriction(FiniteElementSpace self, mfem::ElementDofOrdering e_ordering, mfem::FaceType arg3, mfem::L2FaceValues mul=DoubleValued) -> Operator"""
        return _fespace.FiniteElementSpace_GetFaceRestriction(self, *args, **kwargs)
    GetFaceRestriction = _swig_new_instance_method(_fespace.FiniteElementSpace_GetFaceRestriction)

    def GetQuadratureInterpolator(self, *args):
        r"""
        GetQuadratureInterpolator(FiniteElementSpace self, IntegrationRule ir) -> mfem::QuadratureInterpolator const
        GetQuadratureInterpolator(FiniteElementSpace self, QuadratureSpace qs) -> mfem::QuadratureInterpolator const *
        """
        return _fespace.FiniteElementSpace_GetQuadratureInterpolator(self, *args)
    GetQuadratureInterpolator = _swig_new_instance_method(_fespace.FiniteElementSpace_GetQuadratureInterpolator)

    def GetFaceQuadratureInterpolator(self, ir, type):
        r"""GetFaceQuadratureInterpolator(FiniteElementSpace self, IntegrationRule ir, mfem::FaceType type) -> mfem::FaceQuadratureInterpolator const *"""
        return _fespace.FiniteElementSpace_GetFaceQuadratureInterpolator(self, ir, type)
    GetFaceQuadratureInterpolator = _swig_new_instance_method(_fespace.FiniteElementSpace_GetFaceQuadratureInterpolator)

    def GetVDim(self):
        r"""GetVDim(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetVDim(self)
    GetVDim = _swig_new_instance_method(_fespace.FiniteElementSpace_GetVDim)

    def GetOrder(self, i):
        r"""GetOrder(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetOrder(self, i)
    GetOrder = _swig_new_instance_method(_fespace.FiniteElementSpace_GetOrder)

    def GetFaceOrder(self, i):
        r"""GetFaceOrder(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetFaceOrder(self, i)
    GetFaceOrder = _swig_new_instance_method(_fespace.FiniteElementSpace_GetFaceOrder)

    def GetNDofs(self):
        r"""GetNDofs(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNDofs(self)
    GetNDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNDofs)

    def GetVSize(self):
        r"""GetVSize(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetVSize(self)
    GetVSize = _swig_new_instance_method(_fespace.FiniteElementSpace_GetVSize)

    def GetTrueVSize(self):
        r"""GetTrueVSize(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetTrueVSize(self)
    GetTrueVSize = _swig_new_instance_method(_fespace.FiniteElementSpace_GetTrueVSize)

    def GetNConformingDofs(self):
        r"""GetNConformingDofs(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNConformingDofs(self)
    GetNConformingDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNConformingDofs)

    def GetConformingVSize(self):
        r"""GetConformingVSize(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetConformingVSize(self)
    GetConformingVSize = _swig_new_instance_method(_fespace.FiniteElementSpace_GetConformingVSize)

    def GetOrdering(self):
        r"""GetOrdering(FiniteElementSpace self) -> mfem::Ordering::Type"""
        return _fespace.FiniteElementSpace_GetOrdering(self)
    GetOrdering = _swig_new_instance_method(_fespace.FiniteElementSpace_GetOrdering)

    def FEColl(self):
        r"""FEColl(FiniteElementSpace self) -> FiniteElementCollection"""
        return _fespace.FiniteElementSpace_FEColl(self)
    FEColl = _swig_new_instance_method(_fespace.FiniteElementSpace_FEColl)

    def GetNVDofs(self):
        r"""GetNVDofs(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNVDofs(self)
    GetNVDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNVDofs)

    def GetNEDofs(self):
        r"""GetNEDofs(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNEDofs(self)
    GetNEDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNEDofs)

    def GetNFDofs(self):
        r"""GetNFDofs(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNFDofs(self)
    GetNFDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNFDofs)

    def GetNV(self):
        r"""GetNV(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNV(self)
    GetNV = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNV)

    def GetNE(self):
        r"""GetNE(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNE(self)
    GetNE = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNE)

    def GetNF(self):
        r"""GetNF(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNF(self)
    GetNF = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNF)

    def GetNBE(self):
        r"""GetNBE(FiniteElementSpace self) -> int"""
        return _fespace.FiniteElementSpace_GetNBE(self)
    GetNBE = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNBE)

    def GetNFbyType(self, type):
        r"""GetNFbyType(FiniteElementSpace self, mfem::FaceType type) -> int"""
        return _fespace.FiniteElementSpace_GetNFbyType(self, type)
    GetNFbyType = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNFbyType)

    def GetElementType(self, i):
        r"""GetElementType(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetElementType(self, i)
    GetElementType = _swig_new_instance_method(_fespace.FiniteElementSpace_GetElementType)

    def GetElementVertices(self, i, vertices):
        r"""GetElementVertices(FiniteElementSpace self, int i, intArray vertices)"""
        return _fespace.FiniteElementSpace_GetElementVertices(self, i, vertices)
    GetElementVertices = _swig_new_instance_method(_fespace.FiniteElementSpace_GetElementVertices)

    def GetBdrElementType(self, i):
        r"""GetBdrElementType(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetBdrElementType(self, i)
    GetBdrElementType = _swig_new_instance_method(_fespace.FiniteElementSpace_GetBdrElementType)

    def GetElementTransformation(self, *args):
        r"""
        GetElementTransformation(FiniteElementSpace self, int i) -> ElementTransformation
        GetElementTransformation(FiniteElementSpace self, int i, IsoparametricTransformation ElTr)
        """
        return _fespace.FiniteElementSpace_GetElementTransformation(self, *args)
    GetElementTransformation = _swig_new_instance_method(_fespace.FiniteElementSpace_GetElementTransformation)

    def GetBdrElementTransformation(self, i):
        r"""GetBdrElementTransformation(FiniteElementSpace self, int i) -> ElementTransformation"""
        return _fespace.FiniteElementSpace_GetBdrElementTransformation(self, i)
    GetBdrElementTransformation = _swig_new_instance_method(_fespace.FiniteElementSpace_GetBdrElementTransformation)

    def GetAttribute(self, i):
        r"""GetAttribute(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetAttribute(self, i)
    GetAttribute = _swig_new_instance_method(_fespace.FiniteElementSpace_GetAttribute)

    def GetBdrAttribute(self, i):
        r"""GetBdrAttribute(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetBdrAttribute(self, i)
    GetBdrAttribute = _swig_new_instance_method(_fespace.FiniteElementSpace_GetBdrAttribute)

    def GetElementDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetElementDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetBdrElementDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetBdrElementDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetFaceDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetFaceDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetEdgeDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetEdgeDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetVertexDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetVertexDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetElementInteriorDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetElementInteriorDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetFaceInteriorDofs(self, i, dofs):
        r"""GetFaceInteriorDofs(FiniteElementSpace self, int i, intArray dofs)"""
        return _fespace.FiniteElementSpace_GetFaceInteriorDofs(self, i, dofs)
    GetFaceInteriorDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetFaceInteriorDofs)

    def GetNumElementInteriorDofs(self, i):
        r"""GetNumElementInteriorDofs(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetNumElementInteriorDofs(self, i)
    GetNumElementInteriorDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetNumElementInteriorDofs)

    def GetEdgeInteriorDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetEdgeInteriorDofs(self, i, vdofs)
        return vdofs.ToList()



    def DofsToVDofs(self, *args):
        r"""
        DofsToVDofs(FiniteElementSpace self, intArray dofs, int ndofs=-1)
        DofsToVDofs(FiniteElementSpace self, int vd, intArray dofs, int ndofs=-1)
        """
        return _fespace.FiniteElementSpace_DofsToVDofs(self, *args)
    DofsToVDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_DofsToVDofs)

    def DofToVDof(self, dof, vd, ndofs=-1):
        r"""DofToVDof(FiniteElementSpace self, int dof, int vd, int ndofs=-1) -> int"""
        return _fespace.FiniteElementSpace_DofToVDof(self, dof, vd, ndofs)
    DofToVDof = _swig_new_instance_method(_fespace.FiniteElementSpace_DofToVDof)

    def VDofToDof(self, vdof):
        r"""VDofToDof(FiniteElementSpace self, int vdof) -> int"""
        return _fespace.FiniteElementSpace_VDofToDof(self, vdof)
    VDofToDof = _swig_new_instance_method(_fespace.FiniteElementSpace_VDofToDof)

    @staticmethod
    def AdjustVDofs(vdofs):
        r"""AdjustVDofs(intArray vdofs)"""
        return _fespace.FiniteElementSpace_AdjustVDofs(vdofs)
    AdjustVDofs = _swig_new_static_method(_fespace.FiniteElementSpace_AdjustVDofs)

    def GetElementVDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetElementVDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetBdrElementVDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetBdrElementVDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetFaceVDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetFaceVDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetEdgeVDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetEdgeVDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetVertexVDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetVertexVDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetElementInteriorVDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetElementInteriorVDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetEdgeInteriorVDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _fespace.FiniteElementSpace_GetEdgeInteriorVDofs(self, i, vdofs)
        return vdofs.ToList()



    def RebuildElementToDofTable(self):
        r"""RebuildElementToDofTable(FiniteElementSpace self)"""
        return _fespace.FiniteElementSpace_RebuildElementToDofTable(self)
    RebuildElementToDofTable = _swig_new_instance_method(_fespace.FiniteElementSpace_RebuildElementToDofTable)

    def ReorderElementToDofTable(self):
        r"""ReorderElementToDofTable(FiniteElementSpace self)"""
        return _fespace.FiniteElementSpace_ReorderElementToDofTable(self)
    ReorderElementToDofTable = _swig_new_instance_method(_fespace.FiniteElementSpace_ReorderElementToDofTable)

    def GetElementToDofTable(self):
        r"""GetElementToDofTable(FiniteElementSpace self) -> Table"""
        return _fespace.FiniteElementSpace_GetElementToDofTable(self)
    GetElementToDofTable = _swig_new_instance_method(_fespace.FiniteElementSpace_GetElementToDofTable)

    def GetBdrElementToDofTable(self):
        r"""GetBdrElementToDofTable(FiniteElementSpace self) -> Table"""
        return _fespace.FiniteElementSpace_GetBdrElementToDofTable(self)
    GetBdrElementToDofTable = _swig_new_instance_method(_fespace.FiniteElementSpace_GetBdrElementToDofTable)

    def GetFaceToDofTable(self):
        r"""GetFaceToDofTable(FiniteElementSpace self) -> Table"""
        return _fespace.FiniteElementSpace_GetFaceToDofTable(self)
    GetFaceToDofTable = _swig_new_instance_method(_fespace.FiniteElementSpace_GetFaceToDofTable)

    def BuildDofToArrays(self):
        r"""BuildDofToArrays(FiniteElementSpace self)"""
        return _fespace.FiniteElementSpace_BuildDofToArrays(self)
    BuildDofToArrays = _swig_new_instance_method(_fespace.FiniteElementSpace_BuildDofToArrays)

    def GetElementForDof(self, i):
        r"""GetElementForDof(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetElementForDof(self, i)
    GetElementForDof = _swig_new_instance_method(_fespace.FiniteElementSpace_GetElementForDof)

    def GetLocalDofForDof(self, i):
        r"""GetLocalDofForDof(FiniteElementSpace self, int i) -> int"""
        return _fespace.FiniteElementSpace_GetLocalDofForDof(self, i)
    GetLocalDofForDof = _swig_new_instance_method(_fespace.FiniteElementSpace_GetLocalDofForDof)

    def GetFE(self, i):
        r"""GetFE(FiniteElementSpace self, int i) -> FiniteElement"""
        return _fespace.FiniteElementSpace_GetFE(self, i)
    GetFE = _swig_new_instance_method(_fespace.FiniteElementSpace_GetFE)

    def GetBE(self, i):
        r"""GetBE(FiniteElementSpace self, int i) -> FiniteElement"""
        return _fespace.FiniteElementSpace_GetBE(self, i)
    GetBE = _swig_new_instance_method(_fespace.FiniteElementSpace_GetBE)

    def GetFaceElement(self, i):
        r"""GetFaceElement(FiniteElementSpace self, int i) -> FiniteElement"""
        return _fespace.FiniteElementSpace_GetFaceElement(self, i)
    GetFaceElement = _swig_new_instance_method(_fespace.FiniteElementSpace_GetFaceElement)

    def GetEdgeElement(self, i):
        r"""GetEdgeElement(FiniteElementSpace self, int i) -> FiniteElement"""
        return _fespace.FiniteElementSpace_GetEdgeElement(self, i)
    GetEdgeElement = _swig_new_instance_method(_fespace.FiniteElementSpace_GetEdgeElement)

    def GetTraceElement(self, i, geom_type):
        r"""GetTraceElement(FiniteElementSpace self, int i, mfem::Geometry::Type geom_type) -> FiniteElement"""
        return _fespace.FiniteElementSpace_GetTraceElement(self, i, geom_type)
    GetTraceElement = _swig_new_instance_method(_fespace.FiniteElementSpace_GetTraceElement)

    def GetEssentialVDofs(self, bdr_attr_is_ess, ess_vdofs, component=-1):
        r"""GetEssentialVDofs(FiniteElementSpace self, intArray bdr_attr_is_ess, intArray ess_vdofs, int component=-1)"""
        return _fespace.FiniteElementSpace_GetEssentialVDofs(self, bdr_attr_is_ess, ess_vdofs, component)
    GetEssentialVDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetEssentialVDofs)

    def GetEssentialTrueDofs(self, bdr_attr_is_ess, ess_tdof_list, component=-1):
        r"""GetEssentialTrueDofs(FiniteElementSpace self, intArray bdr_attr_is_ess, intArray ess_tdof_list, int component=-1)"""
        return _fespace.FiniteElementSpace_GetEssentialTrueDofs(self, bdr_attr_is_ess, ess_tdof_list, component)
    GetEssentialTrueDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_GetEssentialTrueDofs)

    @staticmethod
    def MarkerToList(marker, list):
        r"""MarkerToList(intArray marker, intArray list)"""
        return _fespace.FiniteElementSpace_MarkerToList(marker, list)
    MarkerToList = _swig_new_static_method(_fespace.FiniteElementSpace_MarkerToList)

    @staticmethod
    def ListToMarker(list, marker_size, marker, mark_val=-1):
        r"""ListToMarker(intArray list, int marker_size, intArray marker, int mark_val=-1)"""
        return _fespace.FiniteElementSpace_ListToMarker(list, marker_size, marker, mark_val)
    ListToMarker = _swig_new_static_method(_fespace.FiniteElementSpace_ListToMarker)

    def ConvertToConformingVDofs(self, dofs, cdofs):
        r"""ConvertToConformingVDofs(FiniteElementSpace self, intArray dofs, intArray cdofs)"""
        return _fespace.FiniteElementSpace_ConvertToConformingVDofs(self, dofs, cdofs)
    ConvertToConformingVDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_ConvertToConformingVDofs)

    def ConvertFromConformingVDofs(self, cdofs, dofs):
        r"""ConvertFromConformingVDofs(FiniteElementSpace self, intArray cdofs, intArray dofs)"""
        return _fespace.FiniteElementSpace_ConvertFromConformingVDofs(self, cdofs, dofs)
    ConvertFromConformingVDofs = _swig_new_instance_method(_fespace.FiniteElementSpace_ConvertFromConformingVDofs)

    def D2C_GlobalRestrictionMatrix(self, cfes):
        r"""D2C_GlobalRestrictionMatrix(FiniteElementSpace self, FiniteElementSpace cfes) -> SparseMatrix"""
        return _fespace.FiniteElementSpace_D2C_GlobalRestrictionMatrix(self, cfes)
    D2C_GlobalRestrictionMatrix = _swig_new_instance_method(_fespace.FiniteElementSpace_D2C_GlobalRestrictionMatrix)

    def D2Const_GlobalRestrictionMatrix(self, cfes):
        r"""D2Const_GlobalRestrictionMatrix(FiniteElementSpace self, FiniteElementSpace cfes) -> SparseMatrix"""
        return _fespace.FiniteElementSpace_D2Const_GlobalRestrictionMatrix(self, cfes)
    D2Const_GlobalRestrictionMatrix = _swig_new_instance_method(_fespace.FiniteElementSpace_D2Const_GlobalRestrictionMatrix)

    def H2L_GlobalRestrictionMatrix(self, lfes):
        r"""H2L_GlobalRestrictionMatrix(FiniteElementSpace self, FiniteElementSpace lfes) -> SparseMatrix"""
        return _fespace.FiniteElementSpace_H2L_GlobalRestrictionMatrix(self, lfes)
    H2L_GlobalRestrictionMatrix = _swig_new_instance_method(_fespace.FiniteElementSpace_H2L_GlobalRestrictionMatrix)

    def GetTransferOperator(self, coarse_fes, T):
        r"""GetTransferOperator(FiniteElementSpace self, FiniteElementSpace coarse_fes, OperatorHandle T)"""
        return _fespace.FiniteElementSpace_GetTransferOperator(self, coarse_fes, T)
    GetTransferOperator = _swig_new_instance_method(_fespace.FiniteElementSpace_GetTransferOperator)

    def GetTrueTransferOperator(self, coarse_fes, T):
        r"""GetTrueTransferOperator(FiniteElementSpace self, FiniteElementSpace coarse_fes, OperatorHandle T)"""
        return _fespace.FiniteElementSpace_GetTrueTransferOperator(self, coarse_fes, T)
    GetTrueTransferOperator = _swig_new_instance_method(_fespace.FiniteElementSpace_GetTrueTransferOperator)

    def Update(self, want_transform=True):
        r"""Update(FiniteElementSpace self, bool want_transform=True)"""
        return _fespace.FiniteElementSpace_Update(self, want_transform)
    Update = _swig_new_instance_method(_fespace.FiniteElementSpace_Update)

    def GetUpdateOperator(self, *args):
        r"""
        GetUpdateOperator(FiniteElementSpace self) -> Operator
        GetUpdateOperator(FiniteElementSpace self, OperatorHandle T)
        """
        return _fespace.FiniteElementSpace_GetUpdateOperator(self, *args)
    GetUpdateOperator = _swig_new_instance_method(_fespace.FiniteElementSpace_GetUpdateOperator)

    def SetUpdateOperatorOwner(self, own):
        r"""SetUpdateOperatorOwner(FiniteElementSpace self, bool own)"""
        return _fespace.FiniteElementSpace_SetUpdateOperatorOwner(self, own)
    SetUpdateOperatorOwner = _swig_new_instance_method(_fespace.FiniteElementSpace_SetUpdateOperatorOwner)

    def SetUpdateOperatorType(self, tid):
        r"""SetUpdateOperatorType(FiniteElementSpace self, mfem::Operator::Type tid)"""
        return _fespace.FiniteElementSpace_SetUpdateOperatorType(self, tid)
    SetUpdateOperatorType = _swig_new_instance_method(_fespace.FiniteElementSpace_SetUpdateOperatorType)

    def UpdatesFinished(self):
        r"""UpdatesFinished(FiniteElementSpace self)"""
        return _fespace.FiniteElementSpace_UpdatesFinished(self)
    UpdatesFinished = _swig_new_instance_method(_fespace.FiniteElementSpace_UpdatesFinished)

    def GetSequence(self):
        r"""GetSequence(FiniteElementSpace self) -> long"""
        return _fespace.FiniteElementSpace_GetSequence(self)
    GetSequence = _swig_new_instance_method(_fespace.FiniteElementSpace_GetSequence)

    def IsDGSpace(self):
        r"""IsDGSpace(FiniteElementSpace self) -> bool"""
        return _fespace.FiniteElementSpace_IsDGSpace(self)
    IsDGSpace = _swig_new_instance_method(_fespace.FiniteElementSpace_IsDGSpace)

    def Load(self, m, input):
        r"""Load(FiniteElementSpace self, Mesh m, std::istream & input) -> FiniteElementCollection"""
        return _fespace.FiniteElementSpace_Load(self, m, input)
    Load = _swig_new_instance_method(_fespace.FiniteElementSpace_Load)
    __swig_destroy__ = _fespace.delete_FiniteElementSpace

    def Save(self, *args):
        r"""
        Save(FiniteElementSpace self, std::ostream & out)
        Save(FiniteElementSpace self, char const * file, int precision=8)
        Save(FiniteElementSpace self)
        """
        return _fespace.FiniteElementSpace_Save(self, *args)
    Save = _swig_new_instance_method(_fespace.FiniteElementSpace_Save)

# Register FiniteElementSpace in _fespace:
_fespace.FiniteElementSpace_swigregister(FiniteElementSpace)

def FiniteElementSpace_AdjustVDofs(vdofs):
    r"""FiniteElementSpace_AdjustVDofs(intArray vdofs)"""
    return _fespace.FiniteElementSpace_AdjustVDofs(vdofs)
FiniteElementSpace_AdjustVDofs = _fespace.FiniteElementSpace_AdjustVDofs

def FiniteElementSpace_MarkerToList(marker, list):
    r"""FiniteElementSpace_MarkerToList(intArray marker, intArray list)"""
    return _fespace.FiniteElementSpace_MarkerToList(marker, list)
FiniteElementSpace_MarkerToList = _fespace.FiniteElementSpace_MarkerToList

def FiniteElementSpace_ListToMarker(list, marker_size, marker, mark_val=-1):
    r"""FiniteElementSpace_ListToMarker(intArray list, int marker_size, intArray marker, int mark_val=-1)"""
    return _fespace.FiniteElementSpace_ListToMarker(list, marker_size, marker, mark_val)
FiniteElementSpace_ListToMarker = _fespace.FiniteElementSpace_ListToMarker

class QuadratureSpace(object):
    r"""Proxy of C++ mfem::QuadratureSpace class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(QuadratureSpace self, Mesh mesh_, int order_) -> QuadratureSpace
        __init__(QuadratureSpace self, Mesh mesh_, std::istream & _in) -> QuadratureSpace
        """
        _fespace.QuadratureSpace_swiginit(self, _fespace.new_QuadratureSpace(*args))
    __swig_destroy__ = _fespace.delete_QuadratureSpace

    def GetSize(self):
        r"""GetSize(QuadratureSpace self) -> int"""
        return _fespace.QuadratureSpace_GetSize(self)
    GetSize = _swig_new_instance_method(_fespace.QuadratureSpace_GetSize)

    def GetMesh(self):
        r"""GetMesh(QuadratureSpace self) -> Mesh"""
        return _fespace.QuadratureSpace_GetMesh(self)
    GetMesh = _swig_new_instance_method(_fespace.QuadratureSpace_GetMesh)

    def GetNE(self):
        r"""GetNE(QuadratureSpace self) -> int"""
        return _fespace.QuadratureSpace_GetNE(self)
    GetNE = _swig_new_instance_method(_fespace.QuadratureSpace_GetNE)

    def GetElementIntRule(self, idx):
        r"""GetElementIntRule(QuadratureSpace self, int idx) -> IntegrationRule"""
        return _fespace.QuadratureSpace_GetElementIntRule(self, idx)
    GetElementIntRule = _swig_new_instance_method(_fespace.QuadratureSpace_GetElementIntRule)

    def Save(self, *args):
        r"""
        Save(QuadratureSpace self, std::ostream & out)
        Save(QuadratureSpace self, char const * file, int precision=8)
        Save(QuadratureSpace self)
        """
        return _fespace.QuadratureSpace_Save(self, *args)
    Save = _swig_new_instance_method(_fespace.QuadratureSpace_Save)

# Register QuadratureSpace in _fespace:
_fespace.QuadratureSpace_swigregister(QuadratureSpace)

class GridTransfer(object):
    r"""Proxy of C++ mfem::GridTransfer class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _fespace.delete_GridTransfer

    def SetOperatorType(self, type):
        r"""SetOperatorType(GridTransfer self, mfem::Operator::Type type)"""
        return _fespace.GridTransfer_SetOperatorType(self, type)
    SetOperatorType = _swig_new_instance_method(_fespace.GridTransfer_SetOperatorType)

    def ForwardOperator(self):
        r"""ForwardOperator(GridTransfer self) -> Operator"""
        return _fespace.GridTransfer_ForwardOperator(self)
    ForwardOperator = _swig_new_instance_method(_fespace.GridTransfer_ForwardOperator)

    def BackwardOperator(self):
        r"""BackwardOperator(GridTransfer self) -> Operator"""
        return _fespace.GridTransfer_BackwardOperator(self)
    BackwardOperator = _swig_new_instance_method(_fespace.GridTransfer_BackwardOperator)

    def TrueForwardOperator(self):
        r"""TrueForwardOperator(GridTransfer self) -> Operator"""
        return _fespace.GridTransfer_TrueForwardOperator(self)
    TrueForwardOperator = _swig_new_instance_method(_fespace.GridTransfer_TrueForwardOperator)

    def TrueBackwardOperator(self):
        r"""TrueBackwardOperator(GridTransfer self) -> Operator"""
        return _fespace.GridTransfer_TrueBackwardOperator(self)
    TrueBackwardOperator = _swig_new_instance_method(_fespace.GridTransfer_TrueBackwardOperator)

# Register GridTransfer in _fespace:
_fespace.GridTransfer_swigregister(GridTransfer)

class InterpolationGridTransfer(GridTransfer):
    r"""Proxy of C++ mfem::InterpolationGridTransfer class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, coarse_fes, fine_fes):
        r"""__init__(InterpolationGridTransfer self, FiniteElementSpace coarse_fes, FiniteElementSpace fine_fes) -> InterpolationGridTransfer"""
        _fespace.InterpolationGridTransfer_swiginit(self, _fespace.new_InterpolationGridTransfer(coarse_fes, fine_fes))
    __swig_destroy__ = _fespace.delete_InterpolationGridTransfer

    def SetMassIntegrator(self, mass_integ_, own_mass_integ_=True):
        r"""SetMassIntegrator(InterpolationGridTransfer self, BilinearFormIntegrator mass_integ_, bool own_mass_integ_=True)"""
        return _fespace.InterpolationGridTransfer_SetMassIntegrator(self, mass_integ_, own_mass_integ_)
    SetMassIntegrator = _swig_new_instance_method(_fespace.InterpolationGridTransfer_SetMassIntegrator)

    def ForwardOperator(self):
        r"""ForwardOperator(InterpolationGridTransfer self) -> Operator"""
        return _fespace.InterpolationGridTransfer_ForwardOperator(self)
    ForwardOperator = _swig_new_instance_method(_fespace.InterpolationGridTransfer_ForwardOperator)

    def BackwardOperator(self):
        r"""BackwardOperator(InterpolationGridTransfer self) -> Operator"""
        return _fespace.InterpolationGridTransfer_BackwardOperator(self)
    BackwardOperator = _swig_new_instance_method(_fespace.InterpolationGridTransfer_BackwardOperator)

# Register InterpolationGridTransfer in _fespace:
_fespace.InterpolationGridTransfer_swigregister(InterpolationGridTransfer)

class L2ProjectionGridTransfer(GridTransfer):
    r"""Proxy of C++ mfem::L2ProjectionGridTransfer class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, coarse_fes, fine_fes):
        r"""__init__(L2ProjectionGridTransfer self, FiniteElementSpace coarse_fes, FiniteElementSpace fine_fes) -> L2ProjectionGridTransfer"""
        _fespace.L2ProjectionGridTransfer_swiginit(self, _fespace.new_L2ProjectionGridTransfer(coarse_fes, fine_fes))

    def ForwardOperator(self):
        r"""ForwardOperator(L2ProjectionGridTransfer self) -> Operator"""
        return _fespace.L2ProjectionGridTransfer_ForwardOperator(self)
    ForwardOperator = _swig_new_instance_method(_fespace.L2ProjectionGridTransfer_ForwardOperator)

    def BackwardOperator(self):
        r"""BackwardOperator(L2ProjectionGridTransfer self) -> Operator"""
        return _fespace.L2ProjectionGridTransfer_BackwardOperator(self)
    BackwardOperator = _swig_new_instance_method(_fespace.L2ProjectionGridTransfer_BackwardOperator)
    __swig_destroy__ = _fespace.delete_L2ProjectionGridTransfer

# Register L2ProjectionGridTransfer in _fespace:
_fespace.L2ProjectionGridTransfer_swigregister(L2ProjectionGridTransfer)


def UsesTensorBasis(fes):
    r"""UsesTensorBasis(FiniteElementSpace fes) -> bool"""
    return _fespace.UsesTensorBasis(fes)
UsesTensorBasis = _fespace.UsesTensorBasis





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _pbilinearform
else:
    import _pbilinearform

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _pbilinearform.SWIG_PyInstanceMethod_New
_swig_new_static_method = _pbilinearform.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _pbilinearform.MFEM_VERSION
MFEM_VERSION_STRING = _pbilinearform.MFEM_VERSION_STRING
MFEM_VERSION_TYPE = _pbilinearform.MFEM_VERSION_TYPE
MFEM_VERSION_TYPE_RELEASE = _pbilinearform.MFEM_VERSION_TYPE_RELEASE
MFEM_VERSION_TYPE_DEVELOPMENT = _pbilinearform.MFEM_VERSION_TYPE_DEVELOPMENT
MFEM_VERSION_MAJOR = _pbilinearform.MFEM_VERSION_MAJOR
MFEM_VERSION_MINOR = _pbilinearform.MFEM_VERSION_MINOR
MFEM_VERSION_PATCH = _pbilinearform.MFEM_VERSION_PATCH
MFEM_HYPRE_VERSION = _pbilinearform.MFEM_HYPRE_VERSION
import mfem._par.handle
import mfem._par.operators
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.array
import mfem._par.hypre
import mfem._par.globals
import mfem._par.sparsemat
import mfem._par.matrix
import mfem._par.densemat
import mfem._par.bilinearform
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.intrules
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.restriction
import mfem._par.pfespace
import mfem._par.pmesh
import mfem._par.pncmesh
import mfem._par.communication
import mfem._par.sets
class ParBilinearForm(mfem._par.bilinearform.BilinearForm):
    r"""Proxy of C++ mfem::ParBilinearForm class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(ParBilinearForm self, ParFiniteElementSpace pf) -> ParBilinearForm
        __init__(ParBilinearForm self, ParFiniteElementSpace pf, ParBilinearForm bf) -> ParBilinearForm
        """
        _pbilinearform.ParBilinearForm_swiginit(self, _pbilinearform.new_ParBilinearForm(*args))

    def KeepNbrBlock(self, knb=True):
        r"""KeepNbrBlock(ParBilinearForm self, bool knb=True)"""
        return _pbilinearform.ParBilinearForm_KeepNbrBlock(self, knb)
    KeepNbrBlock = _swig_new_instance_method(_pbilinearform.ParBilinearForm_KeepNbrBlock)

    def SetOperatorType(self, tid):
        r"""SetOperatorType(ParBilinearForm self, mfem::Operator::Type tid)"""
        return _pbilinearform.ParBilinearForm_SetOperatorType(self, tid)
    SetOperatorType = _swig_new_instance_method(_pbilinearform.ParBilinearForm_SetOperatorType)

    def Assemble(self, skip_zeros=1):
        r"""Assemble(ParBilinearForm self, int skip_zeros=1)"""
        return _pbilinearform.ParBilinearForm_Assemble(self, skip_zeros)
    Assemble = _swig_new_instance_method(_pbilinearform.ParBilinearForm_Assemble)

    def ParallelAssembleElim(self, *args):
        r"""
        ParallelAssembleElim(ParBilinearForm self) -> HypreParMatrix
        ParallelAssembleElim(ParBilinearForm self, OperatorHandle A_elim)
        """
        return _pbilinearform.ParBilinearForm_ParallelAssembleElim(self, *args)
    ParallelAssembleElim = _swig_new_instance_method(_pbilinearform.ParBilinearForm_ParallelAssembleElim)

    def ParallelAssemble(self, *args):
        r"""
        ParallelAssemble(ParBilinearForm self) -> HypreParMatrix
        ParallelAssemble(ParBilinearForm self, SparseMatrix m) -> HypreParMatrix
        ParallelAssemble(ParBilinearForm self, OperatorHandle A)
        ParallelAssemble(ParBilinearForm self, OperatorHandle A, SparseMatrix A_local)
        """
        return _pbilinearform.ParBilinearForm_ParallelAssemble(self, *args)
    ParallelAssemble = _swig_new_instance_method(_pbilinearform.ParBilinearForm_ParallelAssemble)

    def ParallelEliminateEssentialBC(self, *args):
        r"""
        ParallelEliminateEssentialBC(ParBilinearForm self, intArray bdr_attr_is_ess, HypreParMatrix A, HypreParVector X, HypreParVector B)
        ParallelEliminateEssentialBC(ParBilinearForm self, intArray bdr_attr_is_ess, HypreParMatrix A) -> HypreParMatrix
        """
        return _pbilinearform.ParBilinearForm_ParallelEliminateEssentialBC(self, *args)
    ParallelEliminateEssentialBC = _swig_new_instance_method(_pbilinearform.ParBilinearForm_ParallelEliminateEssentialBC)

    def ParallelEliminateTDofs(self, tdofs_list, A):
        r"""ParallelEliminateTDofs(ParBilinearForm self, intArray tdofs_list, HypreParMatrix A) -> HypreParMatrix"""
        return _pbilinearform.ParBilinearForm_ParallelEliminateTDofs(self, tdofs_list, A)
    ParallelEliminateTDofs = _swig_new_instance_method(_pbilinearform.ParBilinearForm_ParallelEliminateTDofs)

    def TrueAddMult(self, x, y, a=1.0):
        r"""TrueAddMult(ParBilinearForm self, Vector x, Vector y, double const a=1.0)"""
        return _pbilinearform.ParBilinearForm_TrueAddMult(self, x, y, a)
    TrueAddMult = _swig_new_instance_method(_pbilinearform.ParBilinearForm_TrueAddMult)

    def ParFESpace(self):
        r"""ParFESpace(ParBilinearForm self) -> ParFiniteElementSpace"""
        return _pbilinearform.ParBilinearForm_ParFESpace(self)
    ParFESpace = _swig_new_instance_method(_pbilinearform.ParBilinearForm_ParFESpace)

    def SCParFESpace(self):
        r"""SCParFESpace(ParBilinearForm self) -> ParFiniteElementSpace"""
        return _pbilinearform.ParBilinearForm_SCParFESpace(self)
    SCParFESpace = _swig_new_instance_method(_pbilinearform.ParBilinearForm_SCParFESpace)

    def GetProlongation(self):
        r"""GetProlongation(ParBilinearForm self) -> Operator"""
        return _pbilinearform.ParBilinearForm_GetProlongation(self)
    GetProlongation = _swig_new_instance_method(_pbilinearform.ParBilinearForm_GetProlongation)

    def GetRestriction(self):
        r"""GetRestriction(ParBilinearForm self) -> Operator"""
        return _pbilinearform.ParBilinearForm_GetRestriction(self)
    GetRestriction = _swig_new_instance_method(_pbilinearform.ParBilinearForm_GetRestriction)

    def RecoverFEMSolution(self, X, b, x):
        r"""RecoverFEMSolution(ParBilinearForm self, Vector X, Vector b, Vector x)"""
        return _pbilinearform.ParBilinearForm_RecoverFEMSolution(self, X, b, x)
    RecoverFEMSolution = _swig_new_instance_method(_pbilinearform.ParBilinearForm_RecoverFEMSolution)

    def Update(self, nfes=None):
        r"""Update(ParBilinearForm self, FiniteElementSpace nfes=None)"""
        return _pbilinearform.ParBilinearForm_Update(self, nfes)
    Update = _swig_new_instance_method(_pbilinearform.ParBilinearForm_Update)
    __swig_destroy__ = _pbilinearform.delete_ParBilinearForm

    def FormLinearSystem(self, *args):
        r"""
        FormLinearSystem(ParBilinearForm self)
        FormLinearSystem(ParBilinearForm self, intArray ess_tdof_list, Vector x, Vector b, OperatorHandle A, Vector X, Vector B, int copy_interior=0)
        FormLinearSystem(ParBilinearForm self, intArray ess_tdof_list, Vector x, Vector b, SparseMatrix A, Vector X, Vector B, int copy_interior=0)
        FormLinearSystem(ParBilinearForm self, intArray ess_tdof_list, Vector x, Vector b, HypreParMatrix A, Vector X, Vector B, int copy_interior=0)
        """
        return _pbilinearform.ParBilinearForm_FormLinearSystem(self, *args)
    FormLinearSystem = _swig_new_instance_method(_pbilinearform.ParBilinearForm_FormLinearSystem)

    def FormSystemMatrix(self, *args):
        r"""
        FormSystemMatrix(ParBilinearForm self)
        FormSystemMatrix(ParBilinearForm self, intArray ess_tdof_list, OperatorHandle A)
        FormSystemMatrix(ParBilinearForm self, intArray ess_tdof_list, SparseMatrix A)
        FormSystemMatrix(ParBilinearForm self, intArray ess_tdof_list, HypreParMatrix A)
        """
        return _pbilinearform.ParBilinearForm_FormSystemMatrix(self, *args)
    FormSystemMatrix = _swig_new_instance_method(_pbilinearform.ParBilinearForm_FormSystemMatrix)

# Register ParBilinearForm in _pbilinearform:
_pbilinearform.ParBilinearForm_swigregister(ParBilinearForm)

class ParMixedBilinearForm(mfem._par.bilinearform.MixedBilinearForm):
    r"""Proxy of C++ mfem::ParMixedBilinearForm class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(ParMixedBilinearForm self, ParFiniteElementSpace trial_fes, ParFiniteElementSpace test_fes) -> ParMixedBilinearForm
        __init__(ParMixedBilinearForm self, ParFiniteElementSpace trial_fes, ParFiniteElementSpace test_fes, ParMixedBilinearForm mbf) -> ParMixedBilinearForm
        """
        _pbilinearform.ParMixedBilinearForm_swiginit(self, _pbilinearform.new_ParMixedBilinearForm(*args))

    def ParallelAssemble(self, *args):
        r"""
        ParallelAssemble(ParMixedBilinearForm self) -> HypreParMatrix
        ParallelAssemble(ParMixedBilinearForm self, OperatorHandle A)
        """
        return _pbilinearform.ParMixedBilinearForm_ParallelAssemble(self, *args)
    ParallelAssemble = _swig_new_instance_method(_pbilinearform.ParMixedBilinearForm_ParallelAssemble)

    def FormRectangularSystemMatrix(self, trial_tdof_list, test_tdof_list, A):
        r"""FormRectangularSystemMatrix(ParMixedBilinearForm self, intArray trial_tdof_list, intArray test_tdof_list, OperatorHandle A)"""
        return _pbilinearform.ParMixedBilinearForm_FormRectangularSystemMatrix(self, trial_tdof_list, test_tdof_list, A)
    FormRectangularSystemMatrix = _swig_new_instance_method(_pbilinearform.ParMixedBilinearForm_FormRectangularSystemMatrix)

    def FormRectangularLinearSystem(self, trial_tdof_list, test_tdof_list, x, b, A, X, B):
        r"""FormRectangularLinearSystem(ParMixedBilinearForm self, intArray trial_tdof_list, intArray test_tdof_list, Vector x, Vector b, OperatorHandle A, Vector X, Vector B)"""
        return _pbilinearform.ParMixedBilinearForm_FormRectangularLinearSystem(self, trial_tdof_list, test_tdof_list, x, b, A, X, B)
    FormRectangularLinearSystem = _swig_new_instance_method(_pbilinearform.ParMixedBilinearForm_FormRectangularLinearSystem)

    def TrueAddMult(self, x, y, a=1.0):
        r"""TrueAddMult(ParMixedBilinearForm self, Vector x, Vector y, double const a=1.0)"""
        return _pbilinearform.ParMixedBilinearForm_TrueAddMult(self, x, y, a)
    TrueAddMult = _swig_new_instance_method(_pbilinearform.ParMixedBilinearForm_TrueAddMult)
    __swig_destroy__ = _pbilinearform.delete_ParMixedBilinearForm

# Register ParMixedBilinearForm in _pbilinearform:
_pbilinearform.ParMixedBilinearForm_swigregister(ParMixedBilinearForm)

class ParDiscreteLinearOperator(mfem._par.bilinearform.DiscreteLinearOperator):
    r"""Proxy of C++ mfem::ParDiscreteLinearOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, dfes, rfes):
        r"""__init__(ParDiscreteLinearOperator self, ParFiniteElementSpace dfes, ParFiniteElementSpace rfes) -> ParDiscreteLinearOperator"""
        _pbilinearform.ParDiscreteLinearOperator_swiginit(self, _pbilinearform.new_ParDiscreteLinearOperator(dfes, rfes))

    def ParallelAssemble(self):
        r"""ParallelAssemble(ParDiscreteLinearOperator self) -> HypreParMatrix"""
        return _pbilinearform.ParDiscreteLinearOperator_ParallelAssemble(self)
    ParallelAssemble = _swig_new_instance_method(_pbilinearform.ParDiscreteLinearOperator_ParallelAssemble)

    def GetParBlocks(self, blocks):
        r"""GetParBlocks(ParDiscreteLinearOperator self, mfem::Array2D< mfem::HypreParMatrix * > & blocks)"""
        return _pbilinearform.ParDiscreteLinearOperator_GetParBlocks(self, blocks)
    GetParBlocks = _swig_new_instance_method(_pbilinearform.ParDiscreteLinearOperator_GetParBlocks)
    __swig_destroy__ = _pbilinearform.delete_ParDiscreteLinearOperator

# Register ParDiscreteLinearOperator in _pbilinearform:
_pbilinearform.ParDiscreteLinearOperator_swigregister(ParDiscreteLinearOperator)



